local tmr = love.timer
local pi = math.pi
local deg45 = math.deg(45)
local deg180 = math.deg(180)
local deg90 = math.deg(90)

local slideFilter = function(item)
	-- if threeDeeState.clippingMode then return false end
	if item.category == "pushable" then return false end
	if item.colType == "world" then return "cross" end
	if item.colType == "pit" then return "slide" end
	return false
end

local colFunc = function(item, other)
	if other.colType == "collectable" then return "cross" end
	if other.colType == "world" then return "slide" end
	if other.colType == "actor" then
		if not other.playerCollision then
			return "cross"
		else
			return "slide"
		end
	end
	return false
end

behaviorFunctions = {
	initDirection = function(d)
		local dir = d or -math.pi / 2
		if dir == "north" or dir == "n" then			dir = -math.pi / 2
		elseif dir == "south" or dir == "s" then		dir = math.pi / 2
		elseif dir == "east" or dir == "e" then			dir = 0
		elseif dir == "west" or dir == "w" then			dir = -math.pi
		elseif dir == "northwest" or dir == "nw" then	dir = -math.pi + (math.pi / 4)
		elseif dir == "northeast" or dir == "ne" then	dir = -(math.pi / 4)
		elseif dir == "southwest" or dir == "sw" then	dir = (math.pi / 4) * 3
		elseif dir == "southeast" or dir == "se" then	dir = (math.pi / 4)
		end
		return dir
	end,
	defaults = function(self)
		if self.targetX == TARGETNIL or self.targetY == TARGETNIL then
			self.targetX = TARGETNIL
			self.targetY = TARGETNIL
			self.state = "chooseTarget"
			return
		end
		if not self.speed then self.speed = 16 end
		if not self.direction then self.direction = behaviorFunctions.initDirection(0) end
		self.idleTimer = 0
	end,
	collideAndSlideAlongWall = function(self, col, nx, ny, dt)
		local item1, item2 = false, false

		if col.normal.y < 0 then
			local items, len2 = self.parent.colWorld:queryRect(self.x, self.y + self.h + 1, 4, 1, slideFilter)
			if len2 > 0 then item1 = items[1] end
			local items, len2 = self.parent.colWorld:queryRect(self.x + self.w - 4, self.y + self.h + 1, 4, 1, slideFilter)
			if len2 > 0 then item2 = items[1] end

			if item1 and not item2 then nx, ny, _, _ = self.parent.colWorld:move(self, nx + dt * 48, ny, colFunc) end
			if not item1 and item2 then nx, ny, _, _ = self.parent.colWorld:move(self, nx - dt * 48, ny, colFunc) end
		elseif col.normal.y > 0 then
			local items, len2 = self.parent.colWorld:queryRect(self.x, self.y - 1, 4, 1, slideFilter)
			if len2 > 0 then item1 = items[1] end
			local items, len2 = self.parent.colWorld:queryRect(self.x + self.w - 4, self.y - 1, 4, 1, slideFilter)
			if len2 > 0 then item2 = items[1] end

			if item1 and not item2 then nx, ny, _, _ = self.parent.colWorld:move(self, nx + dt * 48, ny, colFunc) end
			if not item1 and item2 then nx, ny, _, _ = self.parent.colWorld:move(self, nx - dt * 48, ny, colFunc) end
		end

		if col.normal.x < 0 then
			local items, len2 = self.parent.colWorld:queryRect(self.x + self.w + 1, self.y, 1, 4, slideFilter)
			if len2 > 0 then item1 = items[1] end
			local items, len2 = self.parent.colWorld:queryRect(self.x + self.w + 1, self.y + self.h - 4, 1, 4, slideFilter)
			if len2 > 0 then item2 = items[1] end

			if item1 and not item2 then nx, ny, _, _ = self.parent.colWorld:move(self, nx, ny + dt * 48, colFunc) end
			if not item1 and item2 then nx, ny, _, _ = self.parent.colWorld:move(self, nx, ny - dt * 48, colFunc) end
		elseif col.normal.x > 0 then
			local items, len2 = self.parent.colWorld:queryRect(self.x - 1, self.y, 1, 4, slideFilter)
			if len2 > 0 then item1 = items[1] end
			local items, len2 = self.parent.colWorld:queryRect(self.x - 1, self.y + self.h - 4, 1, 4, slideFilter)
			if len2 > 0 then item2 = items[1] end

			if item1 and not item2 then nx, ny, _, _ = self.parent.colWorld:move(self, nx, ny + dt * 48, colFunc) end
			if not item1 and item2 then nx, ny, _, _ = self.parent.colWorld:move(self, nx, ny - dt * 48, colFunc) end
		end

		return nx, ny
	end,
}

behaviorStates = {
	idle = {
		set = function(self)
			self.state = "idle"
			self.idleTimer = 1
		end,
		update = function(self, dt)
			self.idleTimer = self.idleTimer - dt
			if self.idleTimer <= 0 then
				self.idleTimer = 0
				self.state = "chooseTarget"
			end
		end
	},
	checkToDamageEntity = {
		update = function(self, dt)

		end
	},
	chooseTarget = {
		update = function(self, dt, colFunc)
			local tx = self.x + math.random(-100, 100)
			local ty = self.y + math.random(-100, 100)
			local cols, len = self.parent.colWorld:project(self, self.x, self.y, self.w, self.h, tx, ty, colFunc)
			for i = 1, len do
				local col = cols[i]
				if col.touch then
					tx = col.touch.x
					ty = col.touch.y
					break
				end
			end

			self._targetX = tx
			self._targetY = ty

			self._targetDistance = math.floor(math_distanceFrom(self.cx, self.cy, self._targetX, self._targetY))
			self.speed = 32
			self._timeItShouldTakeToReachTarget = (self._targetDistance / 32) * 1.5
			self._timeItStartedMoving = tmr.getTime()
			self.state = "moveTowardsTarget"
			-- print(self._timeItShouldTakeToReachTarget)
		end
	},
	moveTowardsTarget = {
		update = function(self, dt, colFunc)
			if self.isDead then return false end
			behaviorFunctions.defaults(self)

			self._targetDistance = math.floor(math_distanceFrom(self.cx, self.cy, self._targetX, self._targetY))
			self._targetAngle = math_findAngle(self._targetX, self._targetY, self.cx, self.cy)

			-- print(self._targetDistance)
			-- print(self.nameID, self.direction, self.speed)

			self.direction = -self._targetAngle + math.rad(90)

			local cx = self.x + math.cos(self.direction) * self.speed * dt
			local cy = self.y + math.sin(self.direction) * self.speed * dt

			-- local ex, ey = self.x + self.w / 2, self.y + self.h / 2

			if self._targetDistance < 20 then
				self.speed = self.speed - dt * 60
				if self.speed <= 0 then
					self.state = "idle"
				end
			end

			local actualX, actualY, cols, len = self.parent.colWorld:move(self, cx, cy, colFunc)
			for i=1, len do
				local col = cols[i]
				local other = col.other

			end

			self.x, self.y = actualX, actualY
			self.cx, self.cy = self.x + self.w / 2, self.y + self.h / 2

			self.spriteDir = math.ceil(math.deg(-(self._targetAngle + deg180 + deg90) % (pi * 2)) / 90)

			if (tmr.getTime() - self._timeItStartedMoving) > self._timeItShouldTakeToReachTarget then
				behaviorStates.idle.set(self)
			end
		end
	},
	moveUntilHitObject = {
		update = function(self, dt, colFunc)
			if not self.speed then self.speed = 32 end
			if not self.direction then self.direction = behaviorFunctions.initDirection(0) end

			local cx = self.x + math.cos(self.direction) * self.speed * dt
			local cy = self.y + math.sin(self.direction) * self.speed * dt

			-- local ex, ey = self.x + self.w / 2, self.y + self.h / 2

			local actualX, actualY, cols, len = self.parent.colWorld:move(self, cx, cy, colFunc)
			for i=1, len do
				local col = cols[i]
				local other = col.other

				if self.kind == "bullet" then
					if other.category == "enemy" or other.destructable or other.hurt or other.activateOnImpact or other.category == "hitbox" then
						if not self.collisionList[other.nameID] then
							other:hurt(self.nameID, 1)
						end
					end
				end

				if col.normal.x < 0 then
					-- print("Touched Right")
				elseif col.normal.x > 0 then
					-- print("Touched Left")
				elseif col.normal.y < 0 then
					-- print("Touched Top")
				elseif col.normal.y > 0 then
					-- print("Touched Bottom")
				end

				if (not self.goesThroughEnemies or other.category == "wall" or other.category == "door") and not other.bulletsPassThrough then
					self.removeThis = true
				end

				if self.kind == "bullet" then
					if other.nameID then
						self.collisionList[other.nameID] = true
					end
				end
			end

			self.x, self.y = actualX, actualY

			-- self.spriteDir = math.ceil(math.deg(-(self.angle + deg180 + deg90) % (pi * 2)) / 90)
		end
	},
	pushedBack = {
		update = function(self, dt, colFunc)
			local cx = self.x + math.cos(self.pushDirection or 0) * self.pushForce * dt
			local cy = self.y + math.sin(self.pushDirection or 0) * self.pushForce * dt

			local actualX, actualY, cols, len = self.parent.colWorld:move(self, cx, cy, colFunc)
			for i=1, len do
				local col = cols[i]
				local other = col.other
			end

			self.x = actualX
			self.y = actualY

			self.pushForce = self.pushForce - dt * 512
			if self.pushForce <= 0 then
				self.pushForce = 0
				self.flashing = 0
				self.state = "idle"
			end
		end
	}
}
