local function error_printer(msg, layer)
	print((debug.traceback("Error: " .. tostring(msg), 1+(layer or 1)):gsub("\n[^\n]+$", "")))
end

function love.errhand(msg)
    local screenshot = love.graphics.newScreenshot()
    screenshot:encode('png', 'crash_ss.png')
    local ss = love.graphics.newImage("crash_ss.png")

   	local eFlux = flux.group()
	msg = tostring(msg)

	error_printer(msg, 2)

	if not love.window or not love.graphics or not love.event then
		return
	end

	if not love.graphics.isCreated() or not love.window.isOpen() then
		local success, status = pcall(love.window.setMode, 800, 600)
		if not success or not status then
			return
		end
	end

	-- Reset state.
	if love.mouse then
		love.mouse.setVisible(true)
		love.mouse.setGrabbed(false)
		love.mouse.setRelativeMode(false)
	end
	if love.joystick then
		-- Stop all joystick vibrations.
		for i,v in ipairs(love.joystick.getJoysticks()) do
			v:setVibration()
		end
	end
	if love.audio then love.audio.stop() end
	love.graphics.reset()
	local font = font.dialog or love.graphics.setNewFont(math.floor(love.window.toPixels(18)))

	love.graphics.setColor(255, 255, 255, 255)

	local trace = debug.traceback()

	love.graphics.origin()

	local err = {}

	table.insert(err, "Error\n")
	table.insert(err, msg.."\n\n")

	for l in string.gmatch(trace, "(.-)\n") do
		if not string.match(l, "boot.lua") then
			l = string.gsub(l, "stack traceback:", "Traceback\n")
			table.insert(err, l)
		end
	end

	local p = table.concat(err, "\n")

	p = string.gsub(p, "\t", "")
	p = string.gsub(p, "%[string \"(.-)\"%]", "%1")

	local e = {
		offset = 200, fade = 0, ix = ss:getWidth(), iy = 0, iscale = 1
	}
	eFlux:to(e, 0.5, {offset = 0, fade = 1, iscale = 0.35, ix = e.ix - 30, iy = 30 }):ease("quartout")

	local function draw()
		eFlux:update(love.timer.getDelta())
		local pos = love.window.toPixels(20)
		love.graphics.setColor(255, 255, 255, 255)
	    love.graphics.draw(ss, 0, 0)
	    love.graphics.setColor(240, 100, 30, 255 * e.fade)
	    love.graphics.rectangle("fill", 0, 0, ss:getWidth(), ss:getHeight())

		love.graphics.setColor(0, 0, 0, 100 * e.fade)
	    love.graphics.draw(ss, e.ix + 5 * love.window.getDPIScale() * e.fade, e.iy + 5 * love.window.getDPIScale() * e.fade, 0, e.iscale, e.iscale, ss:getWidth(), 0)
		love.graphics.setColor(255, 255, 255, 255)
	    love.graphics.draw(ss, e.ix, e.iy, 0, e.iscale, e.iscale, ss:getWidth(), 0)

		love.graphics.push()
		love.graphics.scale(2 * love.window.getDPIScale())
		love.graphics.setFont(font)
		love.graphics.setColor(0,0,0,100 * e.fade)
		love.graphics.printf(p, pos+1, pos+1 + e.offset, love.graphics.getWidth() - pos)
		love.graphics.setColor(255,255,255,255 * e.fade)
		love.graphics.printf(p, pos, pos + e.offset, love.graphics.getWidth() - pos)
		love.graphics.pop()
		love.graphics.present()
	end

	while true do
		love.event.pump()

		for e, a, b, c in love.event.poll() do
			if e == "quit" then
				return
			elseif e == "keypressed" and a == "escape" then
				return
			elseif e == "touchpressed" then
				local name = love.window.getTitle()
				if #name == 0 or name == "Untitled" then name = "Game" end
				local buttons = {"OK", "Cancel"}
				local pressed = love.window.showMessageBox("Quit "..name.."?", "", buttons)
				if pressed == 1 then
					return
				end
			end
		end

		draw()

		if love.timer then
			love.timer.sleep(0.001)
		end
	end
end
