local lgr = love.graphics
local win = love.window
local lfs = love.filesystem
local kbd = love.keyboard
local tmr = love.timer
local lms = love.mouse

local _files = {}
function _files:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function _files:setup(v)
	v = v or {}
	self.x = v.x or 0
	self.y = v.y or 0
	self.w = v.w or 150
	self.h = v.h or 100

	self.parent = v.parent or windowingUIState

	self.directory = v.dir
	print("File List Obj:", self.directory)

	self.scrollY = 0

	self.totalSize = 0
	self.fileCount = 0
	self.contents = {}
	local c = lfs.getDirectoryItems(self.directory)
	for i = 1, #c do
		local info = lfs.getInfo(self.directory.."/"..c[i])
		if c[i] ~= ".DS_Store" then
			local k = info.type
			local n = #self.contents + 1
			local icon
			local path = (self.directory.."/"..c[i])
			path = path:gsub(CUR_GAME.."/drive", "")

			if self.directory:sub(-5) == "/Apps" then
				print(self.directory.."/"..c[i].."/icon.png")
				icon = lgr.newImage(self.directory.."/"..c[i].."/icon.png")
				k = "app"
			end

			self.contents[n] = { name = c[i], kind = k, path = path, selected = false, icon = icon }
			self.fileCount = self.fileCount + 1
			self.totalSize = self.totalSize + info.size
		end
	end
	print(self.totalSize / 1024 / 1024)
	table.sort(self.contents, function(a, b)
		if a.name < b.name then return a.name < b.name end
		return a.kind < b.kind
	end)
	self:resortIcons()
end

function _files:update(dt)

end

function _files:draw()
	lgr.setFont(font.dialog)
	lgr.setColor(0,0,0)
	self.hoveredIcon = 0
	for i, c in pairs(self.contents) do
		lgr.push()
		lgr.translate(c.x, c.y)
		if overlap(self.parent.parent.mouseX, self.parent.parent.mouseY, 1, 1, c.x, c.y, 40, 42) then
			self.hoveredIcon = i--c.name
		end

		if c.selected then
			lgr.setColor(0,0,0,0.4)
			lgr.rectangle("fill", 6, 1, 28, 26, 3)
		end
		if not windowingUIState.icons[c.kind] and c.kind ~= "app" then
			c.kind = "file"
		end
		local icon = c.kind
		local ext = string.lower(string.sub(c.name, -4))
		if ext == ".jpg" or ext == ".gif" or ext == ".png" then
			icon = "image"
		elseif ext == ".txt" then
			icon = "text"
		end

		local icim = windowingUIState.icons[icon]
		if c.kind == "app" then
			icim = c.icon
		end

		lgr.setColor(1,1,1)
		lgr.draw(icim, 8, 2)
		lgr.setFont(font.tiny)
		lgr.setColor(0,0,0)
		lgr.printf(c.name, 0, 40 - 11, 40, "center")
		lgr.pop()
	end

	if self.drawingSelBox then
		self.desktopSelectionBox.x2 = self.parent.parent.mouseX
		self.desktopSelectionBox.y2 = self.parent.parent.mouseY

		local x1, y1 = self.desktopSelectionBox.x1, self.desktopSelectionBox.y1
		local x2, y2 = self.desktopSelectionBox.x2, self.desktopSelectionBox.y2
		if x1 < x2 then
			self.desktopSelectionBox.selX1 = x1
			self.desktopSelectionBox.selX2 = x2 - x1
		else
			self.desktopSelectionBox.selX1 = x2
			self.desktopSelectionBox.selX2 = x1 - x2
		end
		if y1 < y2 then
			self.desktopSelectionBox.selY1 = y1
			self.desktopSelectionBox.selY2 = y2 - y1
		else
			self.desktopSelectionBox.selY1 = y2
			self.desktopSelectionBox.selY2 = y1 - y2
		end

		lgr.setColor(0,0,0,0.12)
		lgr.rectangle("fill", self.desktopSelectionBox.selX1, self.desktopSelectionBox.selY1, self.desktopSelectionBox.selX2, self.desktopSelectionBox.selY2)
		lgr.setColor(0,0,0,0.5)
		lgr.rectangle("line", self.desktopSelectionBox.selX1, self.desktopSelectionBox.selY1, self.desktopSelectionBox.selX2, self.desktopSelectionBox.selY2)

		for i, c in pairs(self.contents) do
			c.selected = false
			if overlap(self.desktopSelectionBox.selX1, self.desktopSelectionBox.selY1, self.desktopSelectionBox.selX2, self.desktopSelectionBox.selY2, c.x, c.y, 40, 42) then
				c.selected = true
			end
		end
	end

	lgr.setColor(0.8,0.8,0.8,0.9)
	lgr.line(self.w - 7, 0, self.w - 7, self.h - 10)
	lgr.setColor(0.8,0.8,0.8,0.4)
	lgr.rectangle("fill", self.w - 7, 0, 7, self.h - 10)

	lgr.setFont(font.tiny)
	lgr.setColor(0.8,0.8,0.8,0.9)
	lgr.rectangle("fill", 0, self.h - 10, self.w, 10)
	lgr.setColor(0,0,0)
	lgr.print(self.fileCount .. " items    " .. math_round(self.totalSize / 1024 / 1024).."MB", 4, self.h - 8)
end

function _files:mousepressed(x, y, button)
	print("File List MP:", x, y, button)

	local selIcon = false
	for i, f in pairs(self.contents) do
		f.selected = false
		if i == self.hoveredIcon then
			f.selected = true
			selIcon = true
		end
	end

	if selIcon then return end

	self.drawingSelBox = true
	self.desktopSelectionBox = {
		x1 = x,
		y1 = y,
		x2 = x,
		y2 = y
	}
end

function _files:mousereleased(x, y, button)
	print("File List MR:", x, y, button)
	if self.drawingSelBox then
		self.drawingSelBox = false
		self.desktopSelectionBox = nil
	end
end

function _files:mousemoved(x, y, dx, dy)
	print("File List MM:", x, y, dx, dy)
end

function _files:doubleclick(x, y, button)
	print("File List DC:", x, y, button)

	if self.contents[self.hoveredIcon] then
		if self.contents[self.hoveredIcon].kind == "directory" then
			windowingUIState:openWindow("folderview", nil, nil, nil, nil, { dir = self.contents[self.hoveredIcon].path })
		elseif self.contents[self.hoveredIcon].kind == "file" then
			local f = self.contents[self.hoveredIcon].name
			local ext = string.lower(string.sub(f, -4))
			if ext == ".jpg" or ext == ".gif" or ext == ".png" then
				windowingUIState:openWindow("imageview", 10, 10, 200, 150, { dir = "", file = CUR_GAME.."/drive/"..self.contents[self.hoveredIcon].path })
				print(ext)
			end
		end
		return
	end
end

function _files:wheelmoved(x, y)
	self.scrollY = self.scrollY + y
end

function _files:focus()

end

function _files:blur()

end

function _files:resize(w, h)
	self.w = w
	self.h = h

	self:resortIcons()
end

function _files:resortIcons()
	self.contentWidth = 0
	local x, y = self.x + 2, self.y + 2
	for i = 1, #self.contents do
		self.contents[i].x = x
		self.contents[i].y = y
		x = x + 40
		if x > self.contentWidth then self.contentWidth = x end
		if x > self.w - 48 then
			y = y + 42
			x = 2
		end
	end
	self.contentHeight = y + 42
end

function _files:maximizeParent()
	return self.parent.parent.x, self.parent.parent.y, self.contentWidth + 12, self.contentHeight + 23
end

return _files
