local lgr = love.graphics
local win = love.window
local lfs = love.filesystem
local kbd = love.keyboard
local tmr = love.timer
local lms = love.mouse

require (CUR_GAME..'.states.windows')

titleState = {
	backgroundUpdate = false,
	backgroundDraw = true,
	name = "Title"
}

TILESIZE = 16
RENDERWIDTH = 320
RENDERHEIGHT = 200

function titleState:load(arg)
	__dprint("Loading Title State")
	local arg = arg or {}
	self.loaded = true

	self:setupMenu()
	stateStack:changeDrawDimensions(320,180)

	self.flux = flux.group()

	self.progress = 0

	self.flux:to(self, 0.5, { progress = 1 }):ease("linear"):oncomplete(function()
		stateStack:changeDrawDimensions(320,180)
		stateStack:replace(windowingUIState)
	end)

	self.logo = lgr.newImage(CUR_GAME.."/images/carrot.png")
end

function titleState:update(dt)
	lms.setCursor(appCursors.normal)

	self.menu:update(dt)
	self.flux:update(dt)
end

function titleState:draw()
	lgr.setColor(0.23,0.31,0.4)
	lgr_fillScreen(0, 0)

	lgr.setColor(1,1,1)
	lgr.draw(self.logo, stateStack.drawWidth / 2, stateStack.drawHeight / 2, 0, 1, 1, 50, 25)

	lgr.rectangle("fill", 100, 150, 120 * self.progress, 8)
	lgr.rectangle("line", 100, 150, 120, 8)

	if self.focused then
		self.menu:draw()
	end
end


function titleState:keypressed(k, r)
	self.menu:keypressed(k, r)
end

function titleState:mousepressed(x, y, button)

end


function titleState:unload()
	__dprint("Unloading Title State")
	self.menu:unload()
	self.menu = nil
	self.loaded = false
	self.vignette = nil
end


function titleState:blur(arg)
	local arg = arg or {}
	self.focused = false
end


function titleState:focus(arg)
	local arg = arg or {}
	self.focused = true
end


function titleState:resize(w, h)
	__dprint("Resize Title State")
end

function titleState:setupMenu()
	self.menu = lfs.load("library/gui/gui.lua")()
	self.menu:init { x = 0, y = 1000, w = 120, h = RENDERHEIGHT - 12, padLeft = 15, padRight = 15, padTop = 6, padBottom = 6, lineHeight = 12, rightWidth = 70,
		defaultFont = font.dialog, menuSelectionFont = font.dialog, topHeader = "Choose Thing", bgColor = {0,0,0,0},
		onescape = function()
			love.event.quit()
		end
	}
	self.menu:addWidget("top", "button", "windows", { label = "Windowing UI", onactivate = function()
		stateStack:changeDrawDimensions(320,180)
		stateStack:replace(windowingUIState)
	end })
end
