local lgr = love.graphics
local win = love.window
local lfs = love.filesystem
local kbd = love.keyboard
local tmr = love.timer
local lms = love.mouse

apps = {}

apps.sample = require (CUR_GAME..'.states.apps.sampleapp')
apps.other = require (CUR_GAME..'.states.apps.otherapp')
apps.noresize = require (CUR_GAME..'.states.apps.noresize')
apps.folderview = require (CUR_GAME..'.states.apps.folderview')
apps.imageview = require (CUR_GAME..'.states.apps.imageview')

genButtonObj = require(CUR_GAME..".library.titleButton")
shrinkyRect = require(CUR_GAME..".library.shrinkyRect")
winObj = require(CUR_GAME..".library.winObj")



windowingUIState = {}

function windowingUIState:load(arg)
	__dprint("Loading Base State")
	local arg = arg or {}
	self.loaded = true
	self.backgroundUpdate = true
	self.backgroundDraw = true
	self.name = "Windows UI"

	self.totalWindows = 0
	self.animObj = {}

	self.windows = {}
	self.clickedWindow = 0

	self:setDesktopPicture()

	self.icons = {}
	self.icons.directory = lgr.newImage(CUR_GAME.."/images/directory.png")
	self.icons.file = lgr.newImage(CUR_GAME.."/images/file.png")
	self.icons.image = lgr.newImage(CUR_GAME.."/images/image.png")
	self.icons.text = lgr.newImage(CUR_GAME.."/images/text.png")

	self.desktopContents = {}
	local c = lfs.getDirectoryItems(CUR_GAME.."/drive")
	for i = 1, #c do
		local info = lfs.getInfo(CUR_GAME.."/drive/"..c[i])
		if c[i] ~= ".DS_Store" then
			k = info.type
			self.desktopContents[#self.desktopContents+1] = { name = c[i], kind = k, selected = false }
		end
	end
	table.sort(self.desktopContents, function(a, b) return a.kind < b.kind end)
	local x, y = stateStack.drawWidth - 40, 12
	for i = 1, #self.desktopContents do
		self.desktopContents[i].x = x
		self.desktopContents[i].y = y
		y = y + 42
		if y > stateStack.drawHeight - 40 then
			y = 12
			x = x - 40
		end
	end


	self:openWindow("noresize", 258, 98, 60, 80)
	self:openWindow("sample", 50, 50, 130, 130)
	self:openWindow("other", 100, 30, 130, 100)
	self:openWindow("imageview", 0, 0, 200, 150, { dir = "", file = CUR_GAME.."/drive/Images/Spelunky Mario.png" })
	self:openWindow("imageview", 10, 10, 200, 150, { dir = "", file = CUR_GAME.."/drive/Images/Luigi.jpg" })
	self:openWindow("folderview", 0, 0, 120, 140, { dir = "images" })
end

function windowingUIState:update(dt)
	lms.setCursor(appCursors.normal)
	self:updateWindows(dt)
	local delObj = {}
	for i, a in pairs(self.animObj) do
		a:update(dt)
		if a.removeThis then
			delObj[#delObj+1] = i
		end
	end

	for i = 1, #delObj do
		self.animObj[delObj[i]] = nil
	end
end

function windowingUIState:draw()
	lgr.setColor(1,1,1)
	lgr.draw(self.desktopPicture, 0, 0)

	lgr.setColor(0,0,0,0.4)
	lgr.rectangle("fill", 0, 0, stateStack.drawWidth, 13)

	-- lgr.push()
	-- lgr.translate(2, 20)
	-- local y = 0
	-- lgr.setFont(font.tiny)
	-- for i, w in pairs(self.windows) do
	-- 	lgr.print(w.nameID, 100, y * 8)
	-- 	y = y + 1
	-- end

	-- local y = 0
	-- for i, w in pairs(self.windowIDCache) do
	-- 	lgr.print(i .. ": " .. w, 0, y * 8)
	-- 	y = y + 1
	-- end
	-- lgr.pop()

	lgr.setFont(font.dialog)
	lgr.setColor(0,0,0)
	self.hoveredDesktopIcon = 0
	for i, c in pairs(self.desktopContents) do
		lgr.push()
		lgr.translate(c.x, c.y)
		if overlap(stateStack.mouseX, stateStack.mouseY, 1, 1, c.x, c.y, 40, 42) then
			self.hoveredDesktopIcon = i--c.name
		end

		if c.selected then
			lgr.setColor(1,1,1,0.4)
			lgr.rectangle("fill", 6, 1, 28, 26, 3)
		end
		if not self.icons[c.kind] then
			c.kind = "file"
		end
		local icon = c.kind
		local ext = string.lower(string.sub(c.name, -4))
		if ext == ".jpg" or ext == ".gif" or ext == ".png" then
			icon = "image"
		elseif ext == ".txt" then
			icon = "text"
		end

		lgr.setColor(1,1,1)
		lgr.draw(self.icons[icon], 8, 2)
		lgr.setFont(font.tiny)
		lgr.setColor(0,0,0)
		lgr.printf(c.name, 0, 40 - 11 + 1, 40, "center")
		lgr.setColor(1,1,1)
		lgr.printf(c.name, 0, 40 - 11, 40, "center")
		lgr.pop()
	end

	if self.drawingSelBox then
		self.desktopSelectionBox.x2 = stateStack.mouseX
		self.desktopSelectionBox.y2 = stateStack.mouseY

		local x1, y1 = self.desktopSelectionBox.x1, self.desktopSelectionBox.y1
		local x2, y2 = self.desktopSelectionBox.x2, self.desktopSelectionBox.y2
		if x1 < x2 then
			self.desktopSelectionBox.selX1 = x1
			self.desktopSelectionBox.selX2 = x2 - x1
		else
			self.desktopSelectionBox.selX1 = x2
			self.desktopSelectionBox.selX2 = x1 - x2
		end
		if y1 < y2 then
			self.desktopSelectionBox.selY1 = y1
			self.desktopSelectionBox.selY2 = y2 - y1
		else
			self.desktopSelectionBox.selY1 = y2
			self.desktopSelectionBox.selY2 = y1 - y2
		end

		lgr.setColor(1,1,1,0.12)
		lgr.rectangle("fill", self.desktopSelectionBox.selX1, self.desktopSelectionBox.selY1, self.desktopSelectionBox.selX2, self.desktopSelectionBox.selY2)
		lgr.setColor(1,1,1)
		lgr.rectangle("line", self.desktopSelectionBox.selX1, self.desktopSelectionBox.selY1, self.desktopSelectionBox.selX2, self.desktopSelectionBox.selY2)

		for i, c in pairs(self.desktopContents) do
			c.selected = false
			if overlap(self.desktopSelectionBox.selX1, self.desktopSelectionBox.selY1, self.desktopSelectionBox.selX2, self.desktopSelectionBox.selY2, c.x, c.y, 40, 42) then
				c.selected = true
			end
		end
	end


	self:drawWindows()

	for i, a in pairs(self.animObj) do
		a:draw()
	end

	lgr.setScissor(0, 0, stateStack.drawWidth, 10)
	lgr.setColor(0,0,0)
	lgr.rectangle("fill", 0, 0, stateStack.drawWidth, 24)
	lgr.setColor(1,1,1)
	lgr.rectangle("fill", 0, 0, stateStack.drawWidth, 24, 6)
	lgr.setScissor()
	lgr.setColor(0,0,0)
	lgr.line(0, 11, stateStack.drawWidth, 11)

	lgr.setFont(font.dialog)
	lgr.setColor(0,0,0)
	lgr.print("File  Edit  View", 24, 1)

	lgr.printf(os.date("%I:%M %m/%d"), 8, 1, stateStack.drawWidth - 16, "right")

	lgr.setColor(0,0.7,0)
	lgr.circle("fill", 12, 5, 4)
	lgr.setColor(1,0.5,0)
	lgr.circle("fill", 14, 5, 3)

	if kbd.isDown("\\") then
		lgr.setColor(1,1,1)
		lgr.printf(stateStack.mouseX .. "," .. stateStack.mouseY, 0, stateStack.drawHeight - 10, stateStack.drawWidth, "right")
	end
end


function windowingUIState:keypressed(k, r)
	if k == "space" then
		self:closeWindow(self:getTopWindowID())
	elseif k == "t" then
		self:tileWindows()
	elseif k == "w" then
		self:openWindow()
	end
end

function windowingUIState:mousepressed(x, y, button)
	self.clickedWindow = 0
	for i = #self.windows, 1, -1 do
		local wn = self.windows[i]
		wn.focused = false
		wn:blur()
		local h = not wn.minimized and wn.h or 12
		if overlap(stateStack.mouseX, stateStack.mouseY, 1, 1, wn.x, wn.y, wn.w, h) then
			print("Clicked Window:", i)
			self.clickedWindow = i
			wn.lastTimeFocused = tmr.getTime()
			break
		end
	end

	if self.clickedWindow > 0 then
		self:sortWindows()
		self.windows[self.clickedWindow]:mousepressed(x, y, button)
		return
	end

	local selIcon = false
	for i, f in pairs(self.desktopContents) do
		f.selected = false
		if i == self.hoveredDesktopIcon then
			f.selected = true
			selIcon = true
		end
	end

	if selIcon then return end

	self.drawingSelBox = true
	self.desktopSelectionBox = {
		x1 = x,
		y1 = y,
		x2 = x,
		y2 = y
	}
end

function windowingUIState:mousereleased(x, y, button)
	if self.clickedWindow > 0 then
		self.windows[self.clickedWindow]:mousereleased(x, y, button)
	end
	self.clickedWindow = 0

	if self.drawingSelBox then
		self.drawingSelBox = false
		self.desktopSelectionBox = nil
	end
end

function windowingUIState:mousemoved(x, y, dx, dy)
	if self.clickedWindow > 0 and self.windows[self.clickedWindow] then
		self.windows[self.clickedWindow]:mousemoved(x, y, dx, dy)
	end
end

function windowingUIState:doubleclick(x, y, button)
	self:mousepressed(x, y, button)
	-- print("DDD", self.clickedWindow)
	if self.clickedWindow > 0 then
		self.windows[self.clickedWindow]:doubleclick(x, y, button)
		return
	end
	-- if self.clickedWindow == 0 then
	-- 	self:openWindow("folderview", nil, nil, nil, nil, { dir = "" })
	-- end

	self:mousereleased(x, y, button)

	if self.desktopContents[self.hoveredDesktopIcon] then
		if self.desktopContents[self.hoveredDesktopIcon].kind == "directory" then
			self:openWindow("folderview", nil, nil, nil, nil, { dir = self.desktopContents[self.hoveredDesktopIcon].name })
		elseif self.desktopContents[self.hoveredDesktopIcon].kind == "file" then
			local f = self.desktopContents[self.hoveredDesktopIcon].name
			local ext = string.lower(string.sub(f, -4))
			if ext == ".jpg" or ext == ".gif" or ext == ".png" then
				self:openWindow("imageview", 10, 10, 200, 150, { dir = "", file = CUR_GAME.."/drive/"..f })
				print(ext)
			end
		end
		return
	end

	self:setDesktopPicture("/Images/afternoon_one.jpg")
end

function windowingUIState:wheelmoved(x, y)
	print(x, y, self.clickedWindow)
	-- self.windows[self.clickedWindow]:wheelmoved(x, y)
end

function windowingUIState:unload()
	self.loaded = false
end


function windowingUIState:blur(arg)
	local arg = arg or {}
	self.focused = false
end


function windowingUIState:focus(arg)
	local arg = arg or {}
	self.focused = true
end


function windowingUIState:resize(w, h)

end

WINSPACING = 10
function windowingUIState:openWindow(app, x, y, w, h, args)
	local x = x or (self.totalWindows % 10) * WINSPACING
	local y = (y or (y or (self.totalWindows % 10) * WINSPACING)) + 12
	local w = w or 200
	local h = h or 130
	local func = function()
		self.windows = self.windows or {}
		-- if self.windows[#self.windows-1] then
			-- x = self.windows[#self.windows-1].x + 16
			-- y = self.windows[#self.windows-1].y + 16
		-- end
		self.windows[#self.windows+1] = winObj:create { parent = self, nameID = "window_"..(self.totalWindows+1), id = (self.totalWindows+1), x = x, y = y, w = w, h = h, app = app, args = args }

		self.totalWindows = self.totalWindows + 1

		self:sortWindows()
	end

	self.animObj[#self.animObj+1] = shrinkyRect:create { x = x + w / 2, y = y + h / 2, w = 0, h = 0, ex = x, ey = y, ew = w, eh = h, endFunc = func }
end

function windowingUIState:closeWindow(id)
	if not self.windowIDCache then return false end
	local n = self.windowIDCache[id]
	if not n then return false end
	if not self.windows[n] then return false end
	-- if self.windows[n].minimized then return false end
	print(n, id)
	local w = self.windows[n]
	self.animObj[#self.animObj+1] = shrinkyRect:create { x = w.x, y = w.y, w = w.w, h = w.h, ex = w.x + w.w / 2, ey = w.y + w.h / 2, ew = 0, eh = 0 }
	self.windows[n] = nil
	self:sortWindows()

end

function windowingUIState:updateWindows(dt)
	for i, w in pairs(self.windows) do
		w:update(dt)
	end
end

function windowingUIState:drawWindows()
	local minWins = {}
	for i, w in pairs(self.windows) do
		w.x = math.floor(w.x)
		w.y = math.floor(w.y)
		w:draw()
		if w.minimized then
			minWins[#minWins+1] = i
		end
	end

	-- lgr.push()
	-- lgr.translate(0, stateStack.drawHeight - 16)
	-- for i = 1, #minWins do
	-- 	lgr.setColor(1,1,1)
	-- 	lgr.rectangle("line", (i-1) * 60 + 2, 0, 58, 14)
	-- 	lgr.print(self.windows[minWins[i]].nameID, (i-1) * 60 + 2 + 2, 2)
	-- end
	-- lgr.pop()
end

function windowingUIState:getTopWindowID()
	if #self.windows == 0 then return false end
	return self.windows[#self.windows].nameID
end

function windowingUIState:sortWindows()
	table.sort(self.windows, function(a, b) return a.lastTimeFocused < b.lastTimeFocused end)

	self.windowIDCache = {}

	for i, w in pairs(self.windows) do
		self.windowIDCache[w.nameID] = i
		w.focused = false
		w:blur()
		if i == #self.windows then
			w.focused = true
			w:focus()
		end
	end
end

function windowingUIState:minimizeWindow(id)
	local _win = self.windows[self.windowIDCache[id]]
	_win.minimized = not _win.minimized
	-- _win.lastTimeFocused = -1000000
	-- self:sortWindows()
end

function windowingUIState:maximizeWindow(id)
	local _win = self.windows[self.windowIDCache[id]]
	local x, y, w, h = -1, 10, (stateStack.drawWidth + 2), (stateStack.drawHeight - 10)
	if _win.context.container then
		if _win.context.container.maximizeParent then
			x, y, w, h = _win.context.container:maximizeParent()
			print("Yes")
		end
	end
	_win.x = x or -1
	_win.y = y or 10
	_win.w = math_clamp(w, 100, stateStack.drawWidth + 2)
	_win.h = math_clamp(h, 100, stateStack.drawHeight - 10)
	_win:resize()
end

function windowingUIState:tileWindows()
	-- print(#self.windows)
	local cols = math.ceil(math.sqrt(#self.windows))
	-- print(cols)
	-- print(colWidth, colHeight)
	local remainder = #self.windows % math.floor(math.sqrt(#self.windows))
	-- print(remainder)

	local tx = 0
	local wn = 0
	for i = 1, cols do
		local colWidth = math.floor(stateStack.drawWidth / cols)
		local colHeight = math.floor(stateStack.drawHeight / cols)
		local rows = math.floor(math.sqrt(#self.windows))
		if i == cols and remainder > 0 then
			colHeight = math.floor(stateStack.drawHeight / remainder)
			rows = remainder
			print("B")
		else
			print("A")
		end

		local ty = 0
		for j = 1, rows do
			wn = wn + 1
			print(wn, colWidth, colHeight)
			local _win = self.windows[wn]
			_win.x = tx * colWidth
			_win.y = ty * colHeight
			_win.w = colWidth
			_win.h = colHeight
			ty = ty + 1
		end
		tx = tx + 1
	end
end

function windowingUIState:setDesktopPicture(img)
	self.desktopPicture = lgr.newCanvas(stateStack.drawWidth, stateStack.drawHeight)

	self.desktopPicture:renderTo(function()
		lgr.setColor(0.23,0.31,0.4)
		lgr_fillScreen(0, 0, 6)
		lgr.setColor(0,0,0,0.2)
		for x = 0, stateStack.drawWidth, 2 do
			for y = 0, stateStack.drawHeight, 2 do
				lgr.rectangle("fill", x, y, 1, 1)
			end
		end
		lgr.push()
		lgr.translate(1, 1)
		for x = 0, stateStack.drawWidth, 2 do
			for y = 0, stateStack.drawHeight, 2 do
				lgr.rectangle("fill", x, y, 1, 1)
			end
		end
		lgr.pop()

		if img then
			print(CUR_GAME.."/drive"..img)
			if lfs.exists(CUR_GAME.."/drive"..img) then
				print("Image!", img)
				lgr.setColor(1,1,1)
				local dp = lgr.newImage(CUR_GAME.."/drive"..img)
				local w, h = dp:getWidth(), dp:getHeight()
				local sx, sy = stateStack.drawWidth / w, stateStack.drawHeight / h
				-- lgr.stencil(function() lgr.rectangle("fill", 0, 0, stateStack.drawWidth, stateStack.drawHeight, 10) end)
				lgr.draw(dp, 0, 0, 0, sx, sy)
				-- lgr.stencil()
			end
		end
	end)
end
