local lgr = love.graphics
local kbd = love.keyboard
local tmr = love.timer

local hitBoxObject = {}

function hitBoxObject:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v)
	return _i
end
function hitBoxObject:setup(v)
	v = v or {}
	self.xo = v.xo or 0
	self.yo = v.yo or 0
	self.w = v.w or 16
	self.h = v.h or 16

	self.owner = v.owner or error("Please provide an owner for this hitbox")

	self.x = self.owner.x + self.xo
	self.y = self.owner.y + self.yo

	self.nameID = v.nameID

	self.category = "hitbox"

	self.colType = "detecthit"

	self.owner.parent.collisionMap:add(self, self.x, self.y, self.w, self.h)
end

function hitBoxObject:hurt(...)
	self.owner:hurt(...)
end

function hitBoxObject:move(x, y, w, h)
	self.owner.parent.collisionMap:update(x, y, w, h)
end

function hitBoxObject:destroy()
	self.owner.parent.collisionMap:remove(self)
end


function addHitBoxObject(entity, name, args)
	if not entity.hitBoxes then entity.hitBoxes = {} end

	args.nameID = name
	args.owner = entity
	args.kind = "hitbox"
	entity.hitBoxes[name] = hitBoxObject:create(args)
end

function moveHitBoxObject(entity, id, nxo, nyo)
	local hb = entity.hitBoxes[id]
	local nxo = nxo or hb.xo
	local nyo = nyo or hb.yo
	local x = entity.x + nxo
	local y = entity.y + nyo
	entity.hitBoxes[id]:move(hb, x, y)
end

function removeHitBoxObject(entity, id)
	if not entity.hitBoxes then return false end
	if not entity.hitBoxes[id] then return false end
	entity.hitBoxes[id]:destroy()
	entity.hitBoxes[id] = nil
end

function removeAllHitBoxObjects(entity)
	for i, h in pairs(entity.hitBoxes) do
		removeHitBoxObject(entity, i)
	end
end
