--[[
	2014-2016 Jason Anderson
	@TheRealAbed on Twitter
	Jasoco on Löve Forum
	The Real Abed on NeoGAF.com

	This is a library I guess. Not really. It doesnt have a license because I dont know anything about licenses. I guess whatever license lets you use it as you wish. I dont claim to own any of this code as it's just code. I did write it myself, but it's nothing special. It also may not work as-is without other stuff or modification, but it probably will actually. Feel free to add your own "kinds" and delete code you want. Just I guess keep my name somewhere or whatever. I dunno.


	How to use:

	1) Require the library in your main.lua as such:
	drawPool = require 'drawPool'

	Or you can load it at any time and as many times as you need like this:
	drawPool = love.filesystem.load("drawPool.lua")()

	This allows you to have as many pools as you need to and layer them separately. For instance, background layers and foreground layers. Example:

	drawPool = {}
	for i = 1, 3 do
		drawPool[i] = love.filesystem.load("drawPool.lua")()
	end

	As long as you run through the drawPool table for each :prepare() and :present() and you :push() the proper objects to the proper drawPool[#] then it'll work perfectly.


	2) Call drawPool:prepare() at the beginning of your :update() function. No arguments are required.


	3) Any time before presenting, use the drawPool:push() function to add objects to the pool. The arguments should be in the form of a table. Use the standard naming conventions like x or y or w or h (Not width or height, unless it's the text kind. (Note: I use "kind" instead of "type" to avoid conflicting with the type() function.) Sorry, it's inconsistent.) And use the layer argument to determine what the sort order is. Remember, pool objects with the same sort order may fight for priority due to how table.sort works. So I find it better to add a tiny random decimal amount to each object's layer to try and make sure they wont fight.

	Examples:
	drawPool:push { kind = "rect", x = 100, y = 100, w = 100, h = 100, color = {255,0,0}, layer = 100 }
	drawPool:push { kind = "circle", x = 200, y = 120, rad = 80, color = {0,0,255}, layer = 95 }
	drawPool:push { kind = "text", x = 50, y = 200, w = 300, align = "center", shadow = true, font = your_font, color = {255,255,255}, layer = 103 }
	drawPool:push { kind = "image", x = 80, y = 60, image = fluffyBunny, quad = bunnyQuad[1], rot = 0, sx = 1, sy = 1, iox = 0, ioy = 0, layer = -304.43 }

	Where sx/sy is Scale and iox/ioy is Image Offset. Quad is optional. Font is optional. Shadow is optional.
	setScissor is not really required in anything. I added it as a hack for my UI system but dont really think I need it now anyway. It's too hacky and clunky. It can probably be removed.
	Colorize requires a shader I made but isnt an important or needed part of the library.


	4) In your :draw() function, call drawPool:present(), but only when you're ready to draw everything. Any :push() calls made after :present() will not happen.


	What happens is every frame, a table is emptied and stuff is added to it, sorted, then iterated through in order to determine what to draw in what order. This lets you basically "draw" any object at any time without worrying whether it's going to be drawn in front of or behind another object. As long as it gets a proper "layer" it'll draw at the right time. You can set layers to really high amounts for certain things (Like UI) and lower amounts for others (Game stuff) for example.

	Also note, this method can create a lot of garbage if used too much each frame. But it's usually not a problem.

	You can use this method to completely replace Löve's drawing frontend if you need to and push every single UI element into a pool if you really wanted to. You can use it for the game graphics or for the UI if you need to. I've used it for flat-shaded and textured polygonal 3D games myself. It was originally concieved for a 2D side-scrolling beat-em-up (Like TMNT) to make sure characters drew in the correct order. Something a lot of NES games in the day didnt bother to do.

	There is officially more text in this README than the entire library.
]]

local lgr = love.graphics

local v = ... or {}
local drawPool = {
	lastPoolCount = 0
}

function drawPool:prepare()
	self.pool = {}
	self.imageCount = nil
	self.rectCount = nil
	self.circleCount = nil
	self.textCount = nil
	self.lineCount = nil
end

function drawPool:push(v)
	if not self.pool then return false end
	if not v.layer then v.layer = 0 end
	if not v.color then v.color = {255,255,255} end
	if v.kind == "text" then
		if not v.font then v.font = lgr.getFont() end
	elseif v.kind == "rect" or v.kind == "circle" then
		if not v.fill then v.fill = "fill" end
		if v.fill == "line" and not v.lineWidth then v.lineWidth = 1 end
	elseif v.kind == "line" then
		if not v.lineWidth then v.lineWidth = 1 end
	end
	self.pool[#self.pool+1] = v
end

function drawPool:image(layer, image, quad, x, y, rot, sx, sy, iox, ioy, kx, ky)
	self:push { kind = "image", image = image, quad = quad, x = x, y = y, rot = rot or 0, sx = sx or 1, sy = sy or 1, iox = iox or 0, ioy = ioy or 0, kx = kx or 0, ky = ky or 0, layer = layer or 0 }
end

function drawPool:rectangle(layer, color, fill, x, y, w, h, rx, ry)
	self:push { kind = "rect", fill = fill, color = color, layer = layer or 0, x = x or 0, y = y or 0, w = w or 1, h = h or 1, rad = rx or nil, rady = ry or rx or nil }
end

function drawPool:setScissor(layer, vars)
	self.pool[#self.pool+1] = {vars = vars, layer = layer or 0, kind = "scissor"}
end

function drawPool:present()
	if not self.pool then return false end

	table.sort(self.pool, function(a, b) return a.layer < b.layer end)

	for i = 1, #self.pool do
		local d = self.pool[i]
		if not d.kind and d.image then d.kind = "image" end
		if not d.kind and d.text then d.kind = "text" end

		if USEPALETTE and d.palette then
			local pal = d.palette or palettes.default
			if type(pal) ~= "table" then pal = palettes[pal] end
			nesShader:send("color1", palette[pal[1]])
			nesShader:send("color2", palette[pal[2]])
			nesShader:send("color3", palette[pal[3]])
			nesShader:send("color4", palette[pal[4]])
			lgr.setShader(nesShader)
		end

		if d.kind == "image" then
			if d.colorize then
				love.graphics.setShader(fogShader)
				fogShader.send(fogShader, "tint", { d.color[1], d.color[2], d.color[3], (d.color[4] or 1) })
			else
				lgr.setColor(d.color or {1,1,1})
			end
			if d.image then
				if type(d.quad) == "table" then
					for i, t in pairs(d.quad) do
						print(i, t)
					end
				end

				if d.outline then
					lgr.setColor(d.outline[1]) -- outline color
					outlineShader:send('pixelsize', { 1 / d.image:getWidth(), 1 / d.image:getHeight() })
					outlineShader:send('size', d.outline[2])
					lgr.setShader(outlineShader)

					if d.quad then
						lgr.draw(d.image, d.quad, d.x, d.y, d.rot or 0, d.sx or 1, d.sy or 1, d.iox or 0, d.ioy or 0, d.kx or 0, d.ky or 0)
					else
						lgr.draw(d.image, d.x, d.y, d.rot or 0, d.sx or 1, d.sy or 1, d.iox or 0, d.ioy or 0, d.kx or 0, d.ky or 0)
					end

					lgr.setShader()
					lgr.setColor(d.color or {1,1,1})
				end

				if d.quad then
					lgr.draw(d.image, d.quad, d.x, d.y, d.rot or 0, d.sx or 1, d.sy or 1, d.iox or 0, d.ioy or 0, d.kx or 0, d.ky or 0)
				else
					lgr.draw(d.image, d.x, d.y, d.rot or 0, d.sx or 1, d.sy or 1, d.iox or 0, d.ioy or 0, d.kx or 0, d.ky or 0)
				end
			else
				print("Image not supplied!")
			end
			if d.colorize then
				love.graphics.setShader()
			end
			self.imageCount = (self.imageCount or 0) + 1
		elseif d.kind == "rect" or d.kind == "rectangle" then
			if d.lineWidth then lgr.setLineWidth(d.lineWidth) end
			lgr.setColor(d.color)
			lgr.rectangle(d.fill or "fill", d.x, d.y, d.w, d.h, d.rad or nil, d.yrad or nil)
			self.rectCount = (self.rectCount or 0) + 1
		elseif d.kind == "line" then
			if d.lineWidth then lgr.setLineWidth(d.lineWidth) end
			lgr.setColor(d.color)
			lgr.line(d.points)
			self.lineCount = (self.lineCount or 0) + 1
		elseif d.kind == "poly" then
			if d.lineWidth then lgr.setLineWidth(d.lineWidth) end
			lgr.setColor(d.color)
			lgr.polygon(d.fill or "fill", d.points)
			self.lineCount = (self.lineCount or 0) + 1
		elseif d.kind == "circle" or d.kind == "circ" then
			if d.lineWidth then lgr.setLineWidth(d.lineWidth) end
			lgr.setColor(d.color)
			lgr.push()
			lgr.translate(d.x, d.y)
			lgr.rotate(d.rot or 0)
			lgr.circle(d.fill or "fill", 0, 0, d.rad, d.detail)
			lgr.pop()
			self.circleCount = (self.circleCount or 0) + 1
		elseif d.kind == "text" then
			local text = d.text or ""
			lgr.setFont(d.font)
			if d.w then
				if d.shadow then
					lgr.setColor(0,0,0)
					lgr.printf(text, d.x+1, d.y+1, d.w, d.align)
				end
				lgr.setColor(d.color)
				lgr.printf(text, d.x, d.y, d.w, d.align)
			else
				if d.shadow then
					lgr.setColor(0,0,0)
					lgr.print(text, d.x+1, d.y+1)
				end
				lgr.setColor(d.color)
				lgr.print(text, d.x, d.y)
			end
			self.textCount = (self.textCount or 0) + 1
		elseif d.kind == "scissor" then
			if d.vars then
				lgr.setScissor(d.vars[1], d.vars[2], d.vars[3], d.vars[4])
			else
				lgr.setScissor()
			end
		end

		if USEPALETTE and d.palette then
			lgr.setShader()
		end
	end
	self.lastPoolCount = #self.pool
end

function drawPool:getCounts()
	return { images = self.imageCount, rects = self.rectCount, lines = self.lineCount, circles = self.circleCount, texts = self.textCount }
end

return drawPool
