local lfs = love.filesystem
local lgr = love.graphics
local win = love.window
local tmr = love.timer
local lms = love.mouse
local mx = lms.getX
local my = lms.getY

local min = math.min
local max = math.max
local floor = math.floor

function chooseNegOrPos()
	local neg = { -1, 1 }
	return neg[math.random(1,2)]
end

function HSL(h, s, l, a)
    if s<=0 then return {l,l,l,a} end
    h, s, l = h/256*6, s/255, l/255
    local c = (1-math.abs(2*l-1))*s
    local x = (1-math.abs(h%2-1))*c
    local m,r,g,b = (l-.5*c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end
    return {(r+m)*255,(g+m)*255,(b+m)*255,a}
end

local __grid = {
	[1] = {
		{1, 0},
		{0, 1}
	},
}
function getGridNoise(s, x, y)
	return __grid[s][math.floor(x % #__grid[s]) + 1][math.floor(y % #__grid[s][1]) + 1]
end

function getNoise(x, y, v, s) return
	math.floor(love.math.noise(x * (s or 1), y * (s or 1)) * (v or 100)) + 1
end

function exportGlobalLog()
	dt = nil
	local transferTable = {}
	local i = 1
	for k, v in pairs(_G) do
		transferTable[i] = { name = k, type = type(v), value = tostring(v) }
		i = i + 1
	end

	table.sort(transferTable, function(a, b)
		-- if a.value < b.value then return a.value < b.value end
		return a.type > b.type
	end)

	local t = "GLOBAL EXPORT LOG:\n"
	for k, v in pairs(transferTable) do
		if not string.find(v.value, "builtin") then
			t = t .. "  " .. v.type .. "  " .. v.name .. "  " .. v.value .. "\n"
		end
	end

	-- t = t .. "\n\narg:"
	-- if arg then
	-- 	for i, a in pairs(arg) do
	-- 		t = t .. tostring(i) .. ", " .. tostring(a) .. "\n"
	-- 	end
	-- end

	lfs.write("global_export_log.txt", t)

end

function copyGrid(oldGrid)
	local newGrid = {}
	for x, gg in pairs(oldGrid) do
		newGrid[x] = {}
		for y, gg in pairs(oldGrid[x]) do
			newGrid[x][y] = oldGrid[x][y]
		end
	end
	return newGrid
end

function newGrid(w, h, val)
	local grid = {}
	for x = 1, w do
		grid[x] = {}
		for y = 1, h do
			grid[x][y] = val
		end
	end
	return grid
end

function printWithShadow(t, x, y, ...)
	local r, g, b, a = lgr.getColor()
	lgr.setColor(0,0,0,a)
	lgr.print(t, x + 1, y + 1, ...)
	lgr.setColor(r,g,b,a)
	lgr.print(t, x, y, ...)
end



-- STRINGS
function string:split(delimiter)
	local result = { }
	local from = 1
	local delim_from, delim_to = string.find( self, delimiter, from )
	while delim_from do
		table.insert( result, string.sub( self, from , delim_from-1 ) )
		from = delim_to + 1
		delim_from, delim_to = string.find( self, delimiter, from )
	end
	table.insert( result, string.sub( self, from ) )
	return result
end

function formatNumber(number) return (string.format("%d", number):reverse():gsub( "(%d%d%d)" , "%1," ):reverse():gsub("^(-?),","%1")) end

function formatTime(t) return os.date("!%X",t) end

function trim(s) return (s:gsub("^%s*(.-)%s*$", "%1")) end

function capitalize(str) return (str:gsub("^%l", string.upper)) end

function firstToUpper(str) return (str:gsub("^%l", string.upper)) end

function createRandomID() return math.random(1000,9999) .. math.random(1000,9999) .. math.random(1000,9999) end

function string_createUUID(n)
    local uuid = ""
    local n = n or 25
    local chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
    for i = 1, n do
        local l = math.random(1, #chars)
        uuid = uuid .. string.sub(chars, l, l)
    end
    return uuid
end

--WORK IN PROGRESS
local __word1 = {
	"one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten",
	"eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen", "eighteen", "ninteen"}
local __word2 = {
	"", "twenty", "thirty", "fourty", "fifty", "sixty", "seventy", "eighty", "ninety"}
function numberWord(num)
	if num <= 20 then
		return __word1[num]
	elseif num > 20 and num < 100 then
		local n1 = (num - 20) / 10
		local n2 = math.floor(n1)
		if n1 == n2 then
			print(n1)
			return __word1[n1]
		else
			local lo = num - ((n2 + 2) * 10)
			return __word2[n2+2] .. "-" .. __word1[lo]
		end
	end
	return num
end


-- MATHS
function math_distanceFrom(x1,y1,x2,y2) return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2) or 0 end

function math_findAngle(x1, y1, x2, y2) return math.atan2(y1 - y2, x2 - x1) - math.pi / 2 end

function math_clamp(n, low, high) return math.max(math.min(n, high), low) end

function overlap(x1,y1,w1,h1, x2,y2,w2,h2)
	if not x1 or not y1 or not w1 or not h1 or not x2 or not y2 or not w2 or not h2 then return false end
	return x1 < x2+w2 and x2 < x1+w1 and y1 < y2+h2 and y2 < y1+h1
end

function math_round(val, decimal)
	if not val then return 0 end
	if (decimal) then
		return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
	else
		return math.floor(val+0.5)
	end
end

local deg90 = math.rad(90)
function returnAngleAmounts(ang, dist)
	local x = math.cos(ang-deg90) * dist
	local y = math.sin(ang-deg90) * dist
	return x, y
end

