local lfs = love.filesystem
local lgr = love.graphics
local win = love.window
local tmr = love.timer
local lms = love.mouse
local mx = lms.getX
local my = lms.getY

local min = math.min
local max = math.max
local floor = math.floor

function lgr_oldColor(r,g,b,a)
	local r, g, b, a = math_round(r/255, 2), math_round(g/255, 2), math_round(b/255, 2), math_round((a or 255)/255, 2)
	lgr.setColor(r,g,b,a)
	print("lgr.setColor("..r..","..g..","..b..","..a..")")
end

function lgr_fillScreen(marginX, marginY, rad)
	lgr.rectangle("fill", marginX or 0, marginY or marginX or 0, stateStack.drawWidth - (marginX or 0) * 2, stateStack.drawHeight - (marginY or marginX or 0) * 2, rad or 0)
end

function isWholeNumber(n)
	return n == math.floor(n)
end

function getRandomSpin()
	return (love.math.random() > 0.5 and 1 or -1) * (math.pi + (love.math.random() * math.pi))
end

function chooseNegOrPos()
	local neg = { -1, 1 }
	return neg[love.math.random(1,2)]
end

function HSL(h, s, l, a)
    if s<=0 then return {l,l,l,a} end
    h, s, l = h/256*6, s/255, l/255
    local c = (1-math.abs(2*l-1))*s
    local x = (1-math.abs(h%2-1))*c
    local m,r,g,b = (l-.5*c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end
    return {(r+m)*255,(g+m)*255,(b+m)*255,a}
end

local __grid = {
	[1] = {
		{1, 0},
		{0, 1}
	},
	[2] = {
		{1, 1, 0},
		{0, 1, 0},
		{0, 1, 1},
		{0, 0, 1},
		{1, 0, 1},
		{1, 0, 0}
	},
	[3] = {
		{0, 1, 1, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 1, 0},
		{0, 1, 1, 0, 0},
		{0, 0, 0, 0, 0},
	},
}
function getGridNoise(s, x, y)
	return __grid[s][math.floor(x % #__grid[s]) + 1][math.floor(y % #__grid[s][1]) + 1]
end

function getNoise(x, y, v, s) return
	math.floor(love.math.noise(x * (s or 1), y * (s or 1)) * (v or 100)) + 1
end

function exportGlobalLog()
	dt = nil
	local transferTable = {}
	local i = 1
	for k, v in pairs(_G) do
		transferTable[i] = { name = k, type = type(v), value = tostring(v) }
		i = i + 1
	end

	table.sort(transferTable, function(a, b)
		return a.type > b.type
	end)

	local t = "GLOBAL EXPORT LOG:\n"
	for k, v in pairs(transferTable) do
		if not string.find(v.value, "builtin") then
			t = t .. "  " .. v.type .. "  " .. v.name .. "  " .. v.value .. "\n"
		end
	end

	lfs.write("global_export_log.txt", t)

end

function isEven(n)
	return ((n % 2) == 0)
end

function copyGrid(oldGrid)
	local newGrid = {}
	for x, gg in pairs(oldGrid) do
		newGrid[x] = {}
		for y, gg in pairs(oldGrid[x]) do
			newGrid[x][y] = oldGrid[x][y]
		end
	end
	return newGrid
end

function newGrid(w, h, val)
	local grid = {}
	for x = 1, w do
		grid[x] = {}
		for y = 1, h do
			grid[x][y] = val
		end
	end
	return grid
end

function printWithShadow(t, x, y, ...)
	local r, g, b, a = lgr.getColor()
	lgr.setColor(0,0,0,a)
	lgr.print(t, x + 1, y + 1, ...)
	lgr.setColor(r,g,b,a)
	lgr.print(t, x, y, ...)
end



-- STRINGS
function string:split(delimiter)
	local result = { }
	local from = 1
	local delim_from, delim_to = string.find( self, delimiter, from )
	while delim_from do
		table.insert( result, string.sub( self, from , delim_from-1 ) )
		from = delim_to + 1
		delim_from, delim_to = string.find( self, delimiter, from )
	end
	table.insert( result, string.sub( self, from ) )
	return result
end

function formatNumber(number) return (string.format("%d", number):reverse():gsub( "(%d%d%d)" , "%1," ):reverse():gsub("^(-?),","%1")) end

function formatTime(t) return os.date("!%X",t) end

function trim(s) return (s:gsub("^%s*(.-)%s*$", "%1")) end

function capitalize(str) return (str:gsub("^%l", string.upper)) end

function firstToUpper(str) return (str:gsub("^%l", string.upper)) end

function createRandomID() return love.math.random(1000,9999) .. love.math.random(1000,9999) .. love.math.random(1000,9999) end

function string_createUUID(n)
    local uuid = ""
    local n = n or 25
    local chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
    for i = 1, n do
        local l = love.math.random(1, #chars)
        uuid = uuid .. string.sub(chars, l, l)
    end
    return uuid
end

--WORK IN PROGRESS
local __word1 = {
	"one", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten",
	"eleven", "twelve", "thirteen", "fourteen", "fifteen", "sixteen", "seventeen", "eighteen", "ninteen"}
local __word2 = {
	"", "twenty", "thirty", "fourty", "fifty", "sixty", "seventy", "eighty", "ninety"}
function numberWord(num)
	if num <= 20 then
		return __word1[num]
	elseif num > 20 and num < 100 then
		local n1 = (num - 20) / 10
		local n2 = math.floor(n1)
		if n1 == n2 then
			__dprint(n1)
			return __word1[n1]
		else
			local lo = num - ((n2 + 2) * 10)
			return __word2[n2+2] .. "-" .. __word1[lo]
		end
	end
	return num
end


-- MATHS
function math_distanceFrom(x1,y1,x2,y2) return math.sqrt((x2 - x1) ^ 2 + (y2 - y1) ^ 2) or 0 end

function math_findAngle(x1, y1, x2, y2) return math.atan2(y1 - y2, x2 - x1) - math.pi / 2 end

function math_clamp(n, low, high) return math.max(math.min(n, high), low) end

function overlap(x1,y1,w1,h1, x2,y2,w2,h2)
	if not x1 or not y1 or not w1 or not h1 or not x2 or not y2 or not w2 or not h2 then return false end
	return x1 < x2+w2 and x2 < x1+w1 and y1 < y2+h2 and y2 < y1+h1
end

function pointInRectangle(x1, y1, x2, y2, w2, h2)
	return x1 >= x2 and x1 < x2 + w2 and y1 >= y2 and y1 < y2 + h2
end

function math_round(val, decimal)
	if not val then return 0 end
	if (decimal) then
		return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
	else
		return math.floor(val+0.5)
	end
end

local deg90 = math.rad(90)
function returnAngleAmounts(ang, dist)
	local x = math.cos(ang-deg90) * dist
	local y = math.sin(ang-deg90) * dist
	return x, y
end


function intersectingSegments(x_a, y_a, x_b, y_b, x_c, y_c, x_d, y_d)
	local ab_x = x_b - x_a
	local ab_y = y_b - y_a

	local cd_x = x_d - x_c
	local cd_y = y_d - y_c

	local c1 = (y_a - y_c) * cd_x - (x_a - x_c) * cd_y
	local c2 = (y_b - y_c) * cd_x - (x_b - x_c) * cd_y
	local c3 = (y_c - y_a) * ab_x - (x_c - x_a) * ab_y
	local c4 = (y_d - y_a) * ab_x - (x_d - x_a) * ab_y

	local colliding = ((c1 < 0) ~= (c2 < 0)) and ((c3 < 0) ~= (c4 < 0))

	local x_i
	local y_i
	if colliding then
		local l = c1 / (c1 - c2)
		x_i = x_a + l * ab_x
		y_i = y_a + l * ab_y
	end

	return colliding, x_i, y_i
end

function addCollisionQuads(self, tl, tr, bl, br)
	if not editorState.loaded then
		local list = {
			tl = tl or false, tr = tr or false,
			bl = bl or false, br = br or false
		}
		local offsets = {
			tl = {0, 0}, tr = {1, 0},
			bl = {0, 1}, br = {1, 1}
		}

		self.colQuads = self.colQuads or {}


		for i, q in pairs(list) do
			if q then
				if not self.colQuads[i] then
					local ox, oy = offsets[i][1] * 8, offsets[i][2] * 8
					self.colQuads[i] = {
						nameID = self.nameID.."_nb", category = self.category or "wall", colType = self.colType or "world", x = self.x + ox, y = self.y + oy, w = 8, h = 8, parent = self.parent }
					addWorldCollision(self.colQuads[i])
				end
			end
		end
		-- if not self.colQuads[1] then
		-- 	self.colQuads[1] = { nameID = self.nameID.."_nb", category = "wall", colType = "world", x = self.x, y = self.y, w = self.w, h = 4, parent = self.parent }
		-- end
	end
end

function removeCollisionQuad(self, quad)
	if self.parent.collisionMap:hasItem(self.colQuads[quad]) then
		self.parent.collisionMap:remove(self.colQuads[quad])
	end
end

function addWorldCollision(self, ox, oy, w, h)
	__dprint("Adding collision box for " .. self.nameID)
	if not editorState.loaded then
		self.parent.collisionMap:add(self, self.x + (ox or 0), self.y + (oy or 0), w or self.w, h or self.h)
		if self.colType == "world" then
			self.parent.mapGridImage:renderTo(function()
				if self.category == "wall" then
					lgr.setColor(0.7,0,0)
				elseif self.category == "water" then
					lgr.setColor(0.2,0.4,1)
				else
					lgr.setColor(0,0,0,0)
				end
				lgr.rectangle("fill", math_round(self.x / 4), math_round(self.y / 4), math_round((w or self.w) / 4), math_round((h or self.h) / 4))
			end)
		end
	end
end

function removeWorldCollisions(self)
	if self.parent.collisionMap:hasItem(self) then
		self.parent.collisionMap:remove(self)
	end

	if self.colBoxes then
		for i, c in pairs(self.colBoxes) do
			if c.parent.collisionMap:hasItem(c) then
				c.parent.collisionMap:remove(c)
			end
		end
	end
	if self.colQuads then
		for i, c in pairs(self.colQuads) do
			if c.parent.collisionMap:hasItem(c) then
				c.parent.collisionMap:remove(c)
			end
		end
	end
end
