local lgr = love.graphics

local _widget = {}

function _widget:create(id, v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(id, v)
	return _i
end
function _widget:setup(id, v)
	self.nameID = id
	self.x = v.x or 0
	self.y = v.y or 0
	self.label = v.label or "<<UNTITLED>>"
	self.subLabel = v.subLabel or ""
	self.parent = v.parent
	self.flux = flux.group()
	self.color = {0,0,0,255}
	self.font = self.parent.defaultFont
	self.bgWidth = 0
	self.h = self.parent.lineHeight

	self.kind = "button"

	self.onactivate = v.onactivate or nil

	self.menuSelectionBGColor = v.menuSelectionBGColor or nil
end

function _widget:update(dt)
	self.flux:update(dt)
end

function _widget:draw()
	lgr.setFont(self.font)

	if self.bgWidth > 0 then
		lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
		lgr.rectangle("fill", self.x, self.y, self.bgWidth * self.parent.w, self.h - 1, 3)
	end

	if self.selected then
		lgr.setFont(self.parent.menuSelectionFont)
	end

	lgr.setColor(self.color)
	lgr.print(self.label, self.x + 4, self.y + 2)
	lgr.printf(self.subLabel, self.x + 4, self.y + 2, self.parent.w - 8, "right")
end

function _widget:keypressed(k, r)
	if k == "return" or k == "space" then
		self:activate()
	end
end

function _widget:activate()
	if self.onactivate then
		self.onactivate(self.value)
	end
end

return _widget
