local lgr = love.graphics

local _widget = {}

function _widget:create(id, v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(id, v)
	return _i
end
function _widget:setup(id, v)
	self.nameID = id
	self.x = v.x or 0
	self.y = v.y or 0
	self.label = v.label or "<<UNTITLED>>"
	self.value = v.value or false
	self.parent = v.parent
	self.flux = flux.group()
	self.color = {0,0,0,255}
	self.font = self.parent.defaultFont
	self.bgWidth = 0
	self.h = self.parent.lineHeight

	self.kind = "checkbox"

	self.checkColor = self.value and {0,255,0,200} or {255,0,0,200}
	self.checkOffset = self.value and 1 or 0

	self.displayValues = v.displayValues or nil

	self.style = v.style or "check"

	self.onactivate = v.onactivate or nil

	self.menuSelectionBGColor = v.menuSelectionBGColor or nil
end

function _widget:update(dt)
	self.flux:update(dt)
end

function _widget:draw()
	lgr.setFont(self.font)

	if self.bgWidth > 0 then
		lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
		lgr.rectangle("fill", self.x, self.y, self.bgWidth * self.parent.w, self.h - 1, 3)
	end

	if self.selected then
		lgr.setFont(self.parent.menuSelectionFont)
	end

	lgr.setColor(self.color)
	lgr.print(self.label, self.x + 4, self.y + 2)
	if self.style == "text" or self.displayValues then
		lgr.printf(self.value and self.displayValues[2] or self.displayValues[1], self.x + 4, self.y + 2, self.parent.w - 8, "right")
	elseif self.style == "toggle" then
		lgr.push()
		lgr.translate(self.x + self.parent.w - 23.5, self.y + 1.5)

		lgr.setColor(self.checkColor)
		lgr.rectangle("fill", 0, 0, 18, 8)

		lgr.setColor(self.color[1], self.color[2], self.color[3], 255 * self.checkOffset)
		lgr.setFont(font.tiny)
		lgr.print("On", 1.5, 1.5)

		lgr.setColor(self.color)
		lgr.rectangle("line", 0, 0, 18, 8)

		lgr.setColor(self.color)
		lgr.rectangle("fill", (self.checkOffset * 10), 1, 9, 7)
		lgr.pop()
	elseif self.style == "check" then
		lgr.push()
		lgr.translate(self.x + self.parent.w - 13.5, self.y + 1.5)
		lgr.rectangle("line", 0, 0, 8, 8)
		if self.value then
			-- lgr.rectangle("fill", 2, 2, 5, 5)
			lgr.line(0, 0, 8, 8)
			lgr.line(8, 0, 0, 8)
		end
		lgr.pop()
	end
end

function _widget:keypressed(k, r)
	if k == "return" or k == "space" then
		self:activate()
	elseif k == "left" then
		self.value = false
		self:animate()
	elseif k == "right" then
		self.value = true
		self:animate()
	end
end

function _widget:activate()
	self.value = not self.value
	self:animate()

	if self.onactivate then
		self.onactivate(self.value)
	end
end

function _widget:animate()
	if self.value then
		self.flux:to(self, 0.5, { checkOffset = 1 }):ease("bounceout")
		self.flux:to(self.checkColor, 0.25, { [1] = 0, [2] = 255 })
	else
		self.flux:to(self, 0.5, { checkOffset = 0 }):ease("bounceout")
		self.flux:to(self.checkColor, 0.25, { [1] = 255, [2] = 0 })
	end
end

return _widget
