local lgr = love.graphics

local _widget = {}

function _widget:create(id, v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(id, v)
	return _i
end
function _widget:setup(id, v)
	self.nameID = id
	self.x = v.x or 0
	self.y = v.y or 0
	self.label = v.label or "<<UNTITLED>>"
	self.parent = v.parent
	self.flux = flux.group()
	self.color = {0,0,0,255}
	self.font = self.parent.defaultFont
	self.bgWidth = 0
	self.h = self.parent.lineHeight

	self.kind = "choice"

	self.value = v.value
	self.choices = v.choices
	self.selection = 1
	for i = 1, #self.choices do
		if self.choices[i] == self.value then
			self.selection = i
			break
		end
	end

	local mw = 0
	for i = 1, #self.choices do
		local nw = self.font:getWidth(self.choices[i])
		if nw > mw then mw = nw end
	end

	self.choiceWidth = self.parent.rightWidth or mw

	self.popupEnabled = false
	self.popupScale = 0

	self.onactivate = v.onactivate or nil
	self.popupWidth = self.choiceWidth + 8
	self.popupHeight = (#self.choices * self.parent.lineHeight) - 1
	self.popupX = self.x + self.parent.w - self.popupWidth
	self.popupY = self.y + 12 + 12

	self.popupCanvas = lgr.newCanvas(self.popupWidth, self.popupHeight, {dpiscale = 1})

	self.menuSelectionBGColor = v.menuSelectionBGColor or nil
end

function _widget:update(dt)
	self.flux:update(dt)
end

function _widget:draw()
	lgr.setFont(self.font)

	if self.bgWidth > 0 then
		lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
		lgr.rectangle("fill", self.x, self.y, self.bgWidth * self.parent.w - (self.popupWidth + 1), self.h - 1, 3)
		lgr.rectangle("fill", self.x + self.parent.w - (self.popupWidth + 1) + 1, self.y, self.bgWidth * self.popupWidth, self.h - 1, 3)
	end

	if self.selected then
		lgr.setFont(self.parent.menuSelectionFont)
	end

	lgr.setColor(self.color)
	lgr.print(self.label, self.x + 4, self.y + 2)

	lgr.printf(self.choices[self.selection], self.x + self.parent.w - self.popupWidth + 4, self.y + 2, self.choiceWidth, "center")
end

function _widget:activate()
	self.value = self.choices[self.selection]
	if self.onactivate then
		self.onactivate(self.value, self.selection)
	end
end

function _widget:keypressed(k, r)
	if not self.popupEnabled then
		if k == "left" then
			self.selection = self.selection - 1
			if self.selection < 1 then self.selection = #self.choices end
			self:activate()
		elseif k == "right" then
			self.selection = self.selection + 1
			if self.selection > #self.choices then self.selection = 1 end
			self:activate()
		elseif k == "return" or k == "space" then
			self:showPopup()
		end
	else
		if k == "up" then
			self.selection = self.selection - 1
			if self.selection < 1 then self.selection = #self.choices end
		elseif k == "down" then
			self.selection = self.selection + 1
			if self.selection > #self.choices then self.selection = 1 end
		elseif k == "return" or k == "space" or k == "escape" then
			self:hidePopup()
		end
	end
end

function _widget:showPopup()
	self.popupEnabled = true
	self.flux:to(self, 0.25, { popupScale = 1 })
end

function _widget:hidePopup()
	self:activate()
	self.flux:to(self, 0.25, { popupScale = 0 }):oncomplete(function() self.popupEnabled = false end)
end

function _widget:drawPopup()
	self.popupCanvas:renderTo(function()
		lgr.clear(self.parent.menuSelectionBGColor[1], self.parent.menuSelectionBGColor[2], self.parent.menuSelectionBGColor[3], 0)
		lgr.push()
		lgr.translate(4, 0)
		lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
		lgr.rectangle("fill", -4, 0, self.popupWidth, ((self.popupHeight - 11) * self.popupScale) + 11, 2)

		lgr.setColor(self.color)
		lgr.rectangle("fill", -4, (self.selection - 1) * (12 * self.popupScale), self.popupWidth, 11, 2)
		for i = 1, #self.choices do
			lgr.setFont(self.font)
			lgr.setColor(self.color)
			if i == self.selection then
				lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
				lgr.setFont(self.parent.menuSelectionFont)
			end
			lgr.printf(self.choices[i], 0, (i - 1) * (12 * self.popupScale) + 2, self.popupWidth - 8, "center")
		end
		lgr.pop()
	end)
end

return _widget
