local lgr = love.graphics

local _widget = {}

function _widget:create(id, v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(id, v)
	return _i
end
function _widget:setup(id, v)
	self.nameID = id
	self.x = v.x or 0
	self.y = v.y or 0
	self.label = v.label or "<<UNTITLED>>"
	self.subLabel = v.subLabel or ""
	self.value = v.value or false
	self.parent = v.parent
	self.flux = flux.group()
	self.color = {0,0,0,255}
	self.font = self.parent.defaultFont
	self.bgWidth = 0
	self.h = self.parent.lineHeight

	self.group = v.group or "default"

	self.kind = "radio"

	self.displayValues = v.displayValues or nil

	self.onactivate = v.onactivate or nil

	self.menuSelectionBGColor = v.menuSelectionBGColor or nil
end

function _widget:update(dt)
	self.flux:update(dt)
end

function _widget:draw()
	lgr.setFont(self.font)

	if self.bgWidth > 0 then
		lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
		lgr.rectangle("fill", self.x, self.y, self.bgWidth * self.parent.w, self.h - 1, 3)
	end

	if self.selected then
		lgr.setFont(self.parent.menuSelectionFont)
	end

	lgr.setColor(self.color)
	lgr.print(self.label, self.x + 4, self.y + 2)
	lgr.printf(self.subLabel, self.x + 4, self.y + 2, self.parent.w - (8 + 12), "right")
	if self.displayValues then
		lgr.printf(self.value and self.displayValues[1] or self.displayValues[2], self.x + 4, self.y + 2, self.parent.w - 8, "right")
	else
		lgr.push()
		lgr.translate(self.x + self.parent.w - 13.5, self.y + 1.5)
		lgr.circle("line", 4, 4, 4.05)
		if self.value then
			lgr.circle("fill", 4, 4, 2.25)
		end
		lgr.pop()
	end
end

function _widget:keypressed(k, r)
	if k == "return" or k == "space" then
		self:activate()
	end
end

function _widget:activate()
	for i, w in pairs(self.parent.page[self.parent.currentPage].widgets) do
		if w.group == self.group then
			w.value = false
			if w.onactivate then
				w.onactivate(w.value)
			end
		end
	end
	self.value = true
	if self.onactivate then
		self.onactivate(self.value)
	end
	self.groupChoice = self.label
end

return _widget
