local lgr = love.graphics

local _widget = {}

function _widget:create(id, v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(id, v)
	return _i
end
function _widget:setup(id, v)
	self.nameID = id
	self.x = v.x or 0
	self.y = v.y or 0
	self.label = v.label or "<<UNTITLED>>"
	self.parent = v.parent
	self.flux = flux.group()
	self.color = {0,0,0,255}
	self.font = self.parent.defaultFont
	self.bgWidth = 0
	self.h = self.parent.lineHeight

	self.kind = "slider"

	self.style = v.style or "fill"

	self.min = v.min or 0
	self.max = v.max or 10

	self.barWidth = self.parent.rightWidth or v.barWidth or 50
	self.segWidth = (self.barWidth - 1) / (self.max - self.min)
	self.offset = -self.min ---math.abs(self.min)

	self.value = math.min(math.max(v.value or 0, self.min), self.max)
	self.animValue = math.floor((self.offset + self.value) * self.segWidth)

	self.increment = v.increment or 1

	self.onactivate = v.onactivate or nil

	self.menuSelectionBGColor = v.menuSelectionBGColor or nil
end

function _widget:update(dt)
	self.flux:update(dt)
end

function _widget:draw()
	lgr.setFont(self.font)

	if self.bgWidth > 0 then
		lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
		lgr.rectangle("fill", self.x, self.y, self.bgWidth * self.parent.w, self.h - 1, 3)
	end

	if self.selected then
		lgr.setFont(self.parent.menuSelectionFont)
	end

	lgr.setColor(self.color)
	lgr.print(self.label, self.x + 4, self.y + 2)

	local v = math.floor(self.animValue)

	if self.bgWidth > 0 and self.style ~= "text" then
		lgr.push()
		lgr.translate(math.min(self.x + v + self.parent.w - (self.barWidth + 4) + 2, self.parent.w - 7), self.y + 2 + 8)
		lgr.scale(self.bgWidth, self.bgWidth)
		lgr.translate(-7, 0)
		lgr.setFont(font.tiny)
		lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
		lgr.rectangle("fill", -1, -1, 15, 7, 2)
		lgr.setColor(self.color)
		lgr.printf(self.value, 0, 0, 15, "center")
		lgr.pop()
	end

	if self.style == "text" then
		lgr.push()
		lgr.translate(self.parent.w - (self.barWidth + 4), self.y + 2)
		lgr.setFont(self.parent.defaultFont)
		lgr.setColor(self.color)
		lgr.printf(self.value, 0, 0, self.barWidth, "center")
		lgr.setFont(font.dialogbold)
		if self.value > self.min then
			lgr.printf("<", 0, 0, self.barWidth, "left")
		end
		if self.value < self.max then
			lgr.printf(">", 0, 0, self.barWidth, "right")
		end
		lgr.pop()
	elseif self.style == "bar" then
		lgr.push()
		lgr.translate(self.parent.w - (self.barWidth + 4), self.y + math.floor((self.h - 1) / 2))
		lgr.setColor(self.color[1], self.color[2], self.color[3], 150)
		lgr.rectangle("fill", 0, 0, self.barWidth, 1)
		lgr.setColor(self.color)
		lgr.rectangle("fill", v, -3, 3, 7)
		lgr.pop()
	elseif self.style == "ball" then
		lgr.push()
		lgr.translate(self.parent.w - (self.barWidth + 4), self.y + math.floor((self.h - 1) / 2))
		lgr.setColor(self.color[1], self.color[2], self.color[3], 150)
		lgr.rectangle("fill", 0, 0, self.barWidth, 1)
		lgr.setColor(self.color)
		lgr.circle("fill", v + 1.5, 0.5, 3.5)
		lgr.pop()
	else
		lgr.push()
		lgr.translate(self.parent.w - (self.barWidth + 4), self.y + math.floor((self.h - 1) / 2))
		lgr.setColor(self.color[1], self.color[2], self.color[3], 150)
		lgr.rectangle("fill", 0, -2, v, 5)
		lgr.setColor(self.color)
		lgr.rectangle("line", 0.5, -1.5, self.barWidth - 1, 4)
		lgr.rectangle("line", 0.5, -1.5, v, 4)
		lgr.pop()
	end
end

function _widget:keypressed(k, r)
	-- if k == "return" or k == "space" then
	-- 	self:activate()
	-- end
end

function _widget:activate()
	if self.onactivate then
		self.onactivate(self.value)
	end
end

function _widget:keypressed(k, r)
	if k == "left" then
		self.value = self.value - self.increment
		if self.value >= self.min then
			self.flux:to(self, 0.20, { animValue = math.floor((self.offset + self.value) * self.segWidth) })
			self:activate()
		else
			self.value = self.min
		end
	elseif k == "right" then
		self.value = self.value + self.increment
		if self.value <= self.max then
			self.flux:to(self, 0.20, { animValue = math.floor((self.offset + self.value) * self.segWidth) })
			self:activate()
		else
			self.value = self.max
		end
	end
end

return _widget
