local lgr = love.graphics
local tmr = love.timer

local _widget = {}

function _widget:create(id, v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(id, v)
	return _i
end
function _widget:setup(id, v)
	self.nameID = id
	self.x = v.x or 0
	self.y = v.y or 0
	self.label = v.label or "<<UNTITLED>>"
	self.value = v.value or ""
	self.parent = v.parent
	self.flux = flux.group()
	self.color = {0,0,0,255}
	self.font = self.parent.defaultFont
	self.bgWidth = 0
	self.h = self.parent.lineHeight

	self.boxWidth = self.parent.rightWidth + 8

	self.kind = "button"

	self.onactivate = v.onactivate or nil

	self.menuSelectionBGColor = v.menuSelectionBGColor or nil
end

function _widget:update(dt)
	self.flux:update(dt)
end

function _widget:draw()
	lgr.setFont(self.font)

	-- if self.bgWidth > 0 then
	-- 	lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
	-- 	lgr.rectangle("fill", self.x, self.y, self.bgWidth * self.parent.w - (self.boxWidth + 1), self.h - 1, 3)
	-- 	lgr.rectangle("fill", self.x + self.parent.w - (self.boxWidth + 1) + 1, self.y, self.bgWidth * self.boxWidth, self.h - 1, 3)
	-- end
	if self.bgWidth > 0 then
		lgr.setColor(self.menuSelectionBGColor or self.parent.menuSelectionBGColor)
		lgr.rectangle("fill", self.x, self.y, self.bgWidth * self.parent.w, self.h - 1, 3)
	end

	if self.selected then
		lgr.setFont(self.parent.menuSelectionFont)
	end

	local caret = (math.sin(tmr.getTime() * 8) >= 0 and self.selected) and "_" or ""

	lgr.setColor(self.color)
	lgr.print(self.label, self.x + 4, self.y + 2)
	lgr.push()
	lgr.translate(self.x + self.parent.w - (self.boxWidth), self.y)
	lgr.setColor(self.color)
	lgr.rectangle("line", 0, 1, self.boxWidth, 10)
	lgr.setColor(self.color)
	lgr.print(self.value .. caret, 2, 2)
	lgr.pop()
end

function _widget:keypressed(k, r)
	if k == "return" or k == "space" then
		self:activate()
	elseif k == "backspace" then
		self.value = string.sub(self.value, 1, -2)
	end
end

function _widget:textinput(t)
	self.value = self.value .. t
	if #self.value > 30 then
		self.value = string.sub(self.value, 1, 30)
	end
end

function _widget:activate()
	if self.onactivate then
		self.onactivate(self.value)
	end
end

return _widget
