local lgr = love.graphics
local lfs = love.filesystem

local lib = {}

local alreadyLoaded = false


function lib:loadImages(folder)
	lgr.setDefaultFilter("nearest", "nearest")

	if not alreadyLoaded then
		self._image = {}
		self._quad = {}
		self._iquad = {}
	end
	local folder = folder or "images"
	local files = lfs.getDirectoryItems(folder)
	local quaddefs = lfs.load(folder.."/quaddefs.lua")()
	for f, i in pairs(files) do
		if lfs.getInfo(folder .. "/" .. i) then
			if string.sub(i, -4) == ".png" then
				local name = string.sub(i, 0, -5)
				self._image[name] = lgr.newImage(folder .. "/" .. i)

				local qs = quaddefs[name] or {16, 16}
				local iw, ih = self._image[name]:getWidth(), self._image[name]:getHeight()

				if qs[3] then
					self._iquad[name] = {}
					for y = 1, ih / qs[2] do
						for x = 1, iw / qs[1] do
							self._iquad[name][#self._iquad[name]+1] = lgr.newQuad((x - 1) * qs[1], (y - 1) * qs[2], qs[1], qs[2], iw, ih)
						end
					end
				else
					self._quad[name] = {}
					for x = 1, iw / qs[1] do
						self._quad[name][x] = {}
						for y = 1, ih / qs[2] do
							self._quad[name][x][y] = lgr.newQuad((x - 1) * qs[1], (y - 1) * qs[2], qs[1], qs[2], iw, ih)
						end
					end
				end
			end
		end
	end
	alreadyLoaded = true
end

function lib:image(name)
	return self._image[name]
end

function lib:quad(name, inline)
	return inline and self._iquad[name] or self._quad[name]
end


return lib
