local win = love.window
local lgr = love.graphics
local tmr = love.timer

local performanceGraph = { enabled = false }

local performanceGraphDrawScale = 1
local performanceGraphUpdateTime = 0
local performanceGraphDrawTime = 0
local performanceGraphPolyGenTime = 0
local performanceGraphSpriteGenTime = 0
local performanceGraphPointCalc = 0
local performanceGraphTextureTime = 0
local performanceGraphTickPos = 0
local performanceGraphDebugDrawTime = 0
local performanceGraphDebugUpdateTime = 0
local performanceGraphTickScale = 3000 / performanceGraphDrawScale
local performanceGraphWidth = 640 / performanceGraphDrawScale
local performanceGraphScale = win.getDPIScale() * 2
local performanceGraphCanvas = lgr.newCanvas(performanceGraphWidth, 800, {dpiscale = 1})
performanceGraphCanvas:setFilter("nearest")

local _performanceGraphTime = 0

local function performanceGraphReset()
	local _s = 1 / 60
	performanceGraphTickPos = 0
	lgr.setCanvas(performanceGraphCanvas)
	lgr.clear(0,0,0,0)
	lgr.setColor(1,1,1,0.333)
	lgr.rectangle("fill", 0, 0, performanceGraphWidth, _s * performanceGraphTickScale)
	lgr.setColor(1,1,1,0.666)
	lgr.line(0, _s * performanceGraphTickScale, performanceGraphWidth, _s * performanceGraphTickScale)
	lgr.setCanvas()
end

-- Call this at the top of love.update before anything else
function performanceGraph:preUpdate()
	if not self.enabled then return false end
	performanceGraphPointCalc = 0
	performanceGraphPolyGenTime = 0
	performanceGraphSpriteGenTime = 0
	performanceGraphDebugDrawTime = 0
	performanceGraphDebugUpdateTime = 0
	_performanceGraphTime = tmr.getTime()
end

-- Call this at the bottom of love.update after everything else
function performanceGraph:postUpdate()
	if not self.enabled then return false end
	performanceGraphUpdateTime = tmr.getTime() - _performanceGraphTime
end

function performanceGraph:addToCalcTime(t)
	if not self.enabled then return false end
	performanceGraphPointCalc = performanceGraphPointCalc + t
end

function performanceGraph:addToTextureTime(t)
	if not self.enabled then return false end
	performanceGraphTextureTime = performanceGraphTextureTime + t
end

function performanceGraph:addToWorldPolyGenTime(t)
	if not self.enabled then return false end
	performanceGraphPolyGenTime = performanceGraphPolyGenTime + t
end

function performanceGraph:addToWorldSpriteGenTime(t)
	if not self.enabled then return false end
	performanceGraphSpriteGenTime = performanceGraphSpriteGenTime + t
end

function performanceGraph:addTodebugUpdateTime(t)
	if not self.enabled then return false end
	performanceGraphDebugUpdateTime = performanceGraphDebugUpdateTime + t
end

function performanceGraph:addTodebugDrawTime(t)
	if not self.enabled then return false end
	performanceGraphDebugDrawTime = performanceGraphDebugDrawTime + t
end

-- Call this at the top of love.draw before anything else
function performanceGraph:preDraw()
	if not self.enabled then return false end
	performanceGraphTextureTime = 0
	_performanceGraphTime = tmr.getTime()
end

-- Call this at the bottom of love.draw after everything else
function performanceGraph:postDraw()
	if not self.enabled then return false end
	performanceGraphDrawTime = tmr.getTime() - _performanceGraphTime

	lgr.setCanvas(performanceGraphCanvas)

	if performanceGraphTickPos == 0 then performanceGraphReset() end
	lgr.setColor(0,1,0)
	lgr.rectangle("fill", performanceGraphTickPos, 0, 1, performanceGraphUpdateTime * performanceGraphTickScale)
	lgr.setColor(0,0.78,0,0.39)
	lgr.rectangle("fill", performanceGraphTickPos, 0, 1, performanceGraphPointCalc * performanceGraphTickScale)
	lgr.setColor(0,0.5,0)
	lgr.rectangle("fill", performanceGraphTickPos, performanceGraphPointCalc * performanceGraphTickScale, 1, performanceGraphPolyGenTime * performanceGraphTickScale)
	lgr.setColor(0.78,0.78,50)
	lgr.rectangle("fill", performanceGraphTickPos, (performanceGraphPolyGenTime + performanceGraphPointCalc) * performanceGraphTickScale, 1, performanceGraphSpriteGenTime * performanceGraphTickScale)

	lgr.setColor(0,0.7,1)
	lgr.rectangle("fill", performanceGraphTickPos, performanceGraphUpdateTime * performanceGraphTickScale, 1, performanceGraphDrawTime * performanceGraphTickScale)
	lgr.setColor(0.2,0.2,0.78)
	lgr.rectangle("fill", performanceGraphTickPos, performanceGraphUpdateTime * performanceGraphTickScale, 1, performanceGraphTextureTime * performanceGraphTickScale)

	lgr.setColor(1,1,0)
	lgr.rectangle("fill", performanceGraphTickPos, love.timer.getDelta() * performanceGraphTickScale, 1, 1)

	local dh = math.floor(performanceGraphDebugDrawTime * performanceGraphTickScale)
	lgr.setColor(0,0.6,0.6,0.78)
	lgr.rectangle("fill", performanceGraphTickPos, performanceGraphUpdateTime * performanceGraphTickScale, 1, dh)

	performanceGraphTickPos = performanceGraphTickPos + 1
	if performanceGraphTickPos >= performanceGraphWidth then performanceGraphTickPos = 0 end

	lgr.setCanvas()

	lgr.setColor(1,1,1)
	lgr.draw(performanceGraphCanvas, 0, lgr.getHeight(), 0, performanceGraphScale * PIXELSCALE * performanceGraphDrawScale, -performanceGraphScale * PIXELSCALE * performanceGraphDrawScale)
end

function performanceGraph:toggle()
	performanceGraphReset()
	self.enabled = not self.enabled
end

return performanceGraph
