local lgr = love.graphics
local win = love.window
local lfs = love.filesystem
local kbd = love.keyboard
local tmr = love.timer
local lms = love.mouse

-- 88""Yb 888888 .dP"Y8 88 8888P 888888 88""Yb        dP     8b    d8  dP"Yb  Yb    dP 888888 88""Yb      dP"Yb  88""Yb  88888 888888  dP""b8 888888
-- 88__dP 88__   `Ybo." 88   dP  88__   88__dP       dP      88b  d88 dP   Yb  Yb  dP  88__   88__dP     dP   Yb 88__dP     88 88__   dP   `"   88
-- 88"Yb  88""   o.`Y8b 88  dP   88""   88"Yb       dP       88YbdP88 Yb   dP   YbdP   88""   88"Yb      Yb   dP 88""Yb o.  88 88""   Yb        88
-- 88  Yb 888888 8bodP' 88 d8888 888888 88  Yb     dP        88 YY 88  YbodP     YP    888888 88  Yb      YbodP  88oodP "bodP' 888888  YboodP   88
local resizerMover = {}

function resizerMover:create(arg)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(arg or {})
	return _i
end
function resizerMover:setup(arg)
	-- editorState._tooltip = ""
	self.x = arg.x
	self.y = arg.y
	self.width = arg.width or 1
	self.height = arg.height or 1

	-- print("Place:", self.x, self.y)

	self.horizontalResize = arg.horizontalResize
	self.verticalResize = arg.verticalResize

	self.expansionAnimation = -2
	self.bgOpacity = 0

	self.movingGrip = ""

	self.entity = arg.entity
	self.entityID = self.entity.nameID
	-- print(">>", self.entityID)

	self.entity.w = self.entity.w or 1
	self.entity.h = self.entity.h or 1
	-- if not self.entity.w then self.entity.w = 1 end

	self.selBoxImage = lgr.newImage(CUR_GAME.."/images/editorui/selbox.png")
	self.selBoxQuad = {
		tl = lgr.newQuad(0, 0, 8, 8, self.selBoxImage:getWidth(), self.selBoxImage:getHeight()),
		bl = lgr.newQuad(0, 8, 8, 8, self.selBoxImage:getWidth(), self.selBoxImage:getHeight()),
		tr = lgr.newQuad(8, 0, 8, 8, self.selBoxImage:getWidth(), self.selBoxImage:getHeight()),
		br = lgr.newQuad(8, 8, 8, 8, self.selBoxImage:getWidth(), self.selBoxImage:getHeight()),
	}

	self.mouseDownTime = 0

	self.grips = {}
	self:setupGrips()
	self.animTween = flux.to(self, 0.15, { expansionAnimation = 2 }):oncomplete(function()
		self.expansionAnimation = 0
	end)

	self.bgTween = flux.to(self, 0.3, { bgOpacity = 1 }):oncomplete(function()
		self.bgOpacity = 0
	end)

	editorState.drawingMarquee = false
	editorState.marqueeBox = nil
end

function resizerMover:update(dt)
	if lms.isDown(1) then
		self.mouseDownTime = self.mouseDownTime + dt
		editorState.drawingMarquee = false
		editorState.marqueeBox = nil
	end
	-- lms.setCursor(appCursors.normal)
	if self.dragging then
		self.x = math_clamp(editorState.mouseMapX - self.offsetX, 0, (editorState.mapData.mapTileWidth + 1) * TILESIZE - self.width)
		self.y = math_clamp(editorState.mouseMapY - self.offsetY, 0, (editorState.mapData.mapTileHeight + 1) * TILESIZE - self.height)
		if self.mouseDownTime > 0.25 then
			lms.setCursor(appCursors.pan)
		end
		return
	end

	for i, g in pairs(self.grips) do
		g.hovering = false
		if pointInRectangle(editorState.mouseMapX, editorState.mouseMapY, g.x - 4, g.y - 4, 8, 8) then
			g.hovering = true
			local c = appCursors.move_horiz
			if i == "top" or i == "bottom" then
				c = appCursors.move_vert
			end
			lms.setCursor(c)
		end

		--  or i == "topleft" or i == "topright" or i == "bottomleft" or i == "bottomright"
		if g.selected then
			if i == "right" or i == "left" or i == "topleft" or i == "topright" or i == "bottomleft" or i == "bottomright" then
				g.x = editorState.mouseMapX
			end
			if i == "left" then
				self.x = g.x
			end

			if i == "top" or i == "bottom" or i == "topleft" or i == "topright" or i == "bottomleft" or i == "bottomright" then
				g.y = editorState.mouseMapY
			end
			if i == "top" then
				self.y = g.y
			end

			g.x = math_clamp(g.x, 0, editorState.mapData.mapTileWidth * TILESIZE)
			g.y = math_clamp(g.y, 0, editorState.mapData.mapTileHeight * TILESIZE)
		end
	end

	if self.horizontalResize then self.width = self.grips["right"].x - self.grips["left"].x end
	if self.verticalResize then self.height = self.grips["bottom"].y - self.grips["top"].y end
end

function resizerMover:draw()
	local borderColor = {0,0,0,0.78}
	local e = self.expansionAnimation
	editorState.uiPool:push { color = {1,1,0,0.39*self.bgOpacity}, x = self.x - 0.5 - e, y = self.y - 0.5 - e, w = self.width + 1 + e * 2, h = self.height + 1 + e * 2, layer = 119900, kind = "rect", fill = "fill" }
	-- editorState.uiPool:push { color = {1,1,0,255}, x = self.x - 0.5 - e, y = self.y - 0.5 - e, w = self.width + 1 + e * 2, h = self.height + 1 + e * 2, layer = 119900, kind = "rect", fill = "line" }
	-- editorState.uiPool:push { color = {255,70,0}, x = self.x - 1.5 - e, y = self.y - 1.5 - e, w = self.width + 3 + e * 2, h = self.height + 3 + e * 2, layer = 119899, kind = "rect", fill = "line" }

	self:drawSelectionBox(self.x / TILESIZE, self.y / TILESIZE, self.width / TILESIZE, self.height / TILESIZE, {1,0.78,0})

	if not self.dragging and not self.animating then
		for i, g in pairs(self.grips) do
			local c = {0,1,0}
			if g.selected then
				c = {0,0,1}
			elseif g.hovering then
				c = {0,1,1}
			end
			local x, y, w, h = math.floor(g.x - 3), math.floor(g.y - 3), 6, 6
			-- editorState.uiPool:push { color = {0,0,0,0.78}, x = x-1, y = y-1, w = w+2, h = h+2, layer = 120000, kind = "rect", fill = "fill" }
			-- editorState.uiPool:push { color = c, x = x, y = y, w = w, h = h, layer = 120000.1, kind = "rect", fill = "fill" }

			local t = math.cos(tmr.getTime() * 5) / 2
			editorState.uiPool:push { color = {c[1] * 0.3, c[2] * 0.3, c[3] * 0.3}, x = g.x, y = g.y, rad = 7 + t, detail = 3, rot = g.rot, layer = 120000, kind = "circle", fill = "fill" }
			editorState.uiPool:push { color = c, x = g.x, y = g.y, rad = 5 + t, detail = 3, rot = g.rot, layer = 120000.1, kind = "circle", fill = "fill" }

			-- editorState.uiPool:push { color = {0,0,0}, x = g.x, y = g.y, rad = 5, layer = 120000, kind = "circle", fill = "fill" }
			-- editorState.uiPool:push { color = c, x = g.x, y = g.y, rad = 4, layer = 120000.1, kind = "circle", fill = "fill" }
		end
	end
end

function resizerMover:mousepressed(x, y, button)
	if kbd.isDown("lalt", "ralt") then
		editorState.resizerMover = nil
	end

	-- print("Press", editorState.mouseMapX, editorState.mouseMapY)
	for i, g in pairs(self.grips) do
		if pointInRectangle(editorState.mouseMapX, editorState.mouseMapY, g.x - 4, g.y - 4, 8, 8) and button == 1 then
			self.movingGrip = i
			g.selected = true
		end
	end
	if self.movingGrip == "" then
		if pointInRectangle(editorState.mouseMapX, editorState.mouseMapY, self.x, self.y, self.width, self.height) and button == 1 then
			self.offsetX = editorState.mouseMapX - self.x
			self.offsetY = editorState.mouseMapY - self.y
			self.dragging = true
		end
	end

	if not pointInRectangle(editorState.mouseMapX, editorState.mouseMapY, self.x - 5, self.y - 5, self.width + 10, self.height + 10) and button == 1 then
		editorState.resizerMover = nil
	end
end

function resizerMover:mousereleased(x, y, button)
	editorState.drawingMarquee = false
	editorState.marqueeBox = nil

	self.mouseDownTime = 0

	if editorState.currentLayer == "entities" then
		-- if pointInRectangle(x, y, editorState.toolbars.editor.children["delete"].x, editorState.toolbars.editor.children["delete"].y, editorState.toolbars.editor.children["delete"].w, editorState.toolbars.editor.children["delete"].h) and button == 1 and self.dragging then
		-- 	editorState:deleteWorldEntity(editorState.currentEntityID)
		-- 	editorState.fileIsDirty = true
		-- 	return
		-- end
	else
		editorState.resizerMover = nil
		self.dragging = false
	end

	if self.dragging then
		self.x = math.floor((self.x / 8) + 0.5) * 8
		self.y = math.floor((self.y / 8) + 0.5) * 8

		self:setupGrips()

		self.dragging = false
	else
		for i, g in pairs(self.grips) do
			g.x = math.floor((g.x / TILESIZE) + 0.5) * TILESIZE
			g.y = math.floor((g.y / TILESIZE) + 0.5) * TILESIZE
		end

		if self.horizontalResize then
			self.x = self.grips["left"].x
			self.width = math.max(self.grips["right"].x - self.grips["left"].x, TILESIZE)
		end

		if self.verticalResize then
			self.y = self.grips["top"].y
			self.height = math.max(self.grips["bottom"].y - self.grips["top"].y, TILESIZE)
		end

		self:setupGrips()

		for i, g in pairs(self.grips) do
			self.movingGrip = ""
			g.selected = false
		end
	end

	local rx, ry, rw, rh = self.x, self.y, math.max(self.width / TILESIZE, 0.5), math.max(self.height / TILESIZE, 0.5)
	-- if not editorState:checkForEntity(self.entity.nameID, self.x / TILESIZE, self.y / TILESIZE) then
	-- 	if self.entity.x ~= rx or self.entity.y ~= ry or self.entity.w ~= rw or self.entity.h ~= rh then
	-- 		local n = #editorState.undoStack+1
	-- 		editorState.undoStack[n] = {
	-- 			task = "moveEntity",
	-- 			id = self.entity.nameID,
	-- 			pCoords = { x = self.entity.x, y = self.entity.y, w = self.entity.w, h = self.entity.h }
	-- 		}
	-- 	end

	-- print(rx, ry, rw, rh)

	self.entity.x = rx
	self.entity.y = ry
	self.entity.w = rw
	self.entity.h = rh
	editorState.fileIsDirty = true

	-- else
		-- self.animating = true
		-- self.tween = flux.to(self, 0.15, { x = self.entity.x * TILESIZE, y = self.entity.y * TILESIZE }):oncomplete(function() self:setupGrips() self.animating = false end)
	-- end
end

function resizerMover:mousemoved(x, y, dx, dy)
	self.mouseDownTime = 0.5
end

function resizerMover:setupGrips()
	if self.horizontalResize then
		self.grips["left"] = { x = self.x, y = self.y + self.height / 2, rot = math.pi }
		self.grips["right"] = { x = self.x + self.width, y = self.y + self.height / 2, rot = 0 }
	end

	if self.verticalResize then
		self.grips["top"] = { x = self.x + self.width / 2, y = self.y, rot = math.pi * 1.5 }
		self.grips["bottom"] = { x = self.x + self.width / 2, y = self.y + self.height, rot = math.pi * 0.5 }
	end

	-- if self.horizontalResize and self.verticalResize then
	-- 	self.grips["topleft"] = { x = self.x, y = self.y }
	-- 	self.grips["topright"] = { x = self.x + self.width, y = self.y }
	-- 	self.grips["bottomleft"] = { x = self.x, y = self.y + self.height }
	-- 	self.grips["bottomright"] = { x = self.x + self.width, y = self.y + self.height }
	-- end
end

function resizerMover:drawSelectionBox(x, y, w, h)
	local c = {1,1,0}

	local o = math.min(math.cos(tmr.getTime() * 10), 0) - 1

	-- print("Drawing", x, y)

	editorState.uiPool:push {
		image = self.selBoxImage, quad = self.selBoxQuad.tl, color = c, x = x * TILESIZE, y = y * TILESIZE,
		layer = 110001+y, rot = 0, sx = 1, sy = 1, iox = 1 - o, ioy = 1 - o, kind = "image" }

	editorState.uiPool:push {
		image = self.selBoxImage, quad = self.selBoxQuad.tr, color = c, x = x * TILESIZE + w * TILESIZE, y = y * TILESIZE,
		layer = 110002+y, rot = 0, sx = 1, sy = 1, iox = 7 + o, ioy = 1 - o, kind = "image" }

	editorState.uiPool:push {
		image = self.selBoxImage, quad = self.selBoxQuad.bl, color = c, x = x * TILESIZE, y = y * TILESIZE + h * TILESIZE,
		layer = 110004+y, rot = 0, sx = 1, sy = 1, iox = 1 - o, ioy = 7 + o, kind = "image" }

	editorState.uiPool:push {
		image = self.selBoxImage, quad = self.selBoxQuad.br, color = c, x = x * TILESIZE + w * TILESIZE, y = y * TILESIZE + h * TILESIZE,
		layer = 110003+y, rot = 0, sx = 1, sy = 1, iox = 7 + o, ioy = 7 + o, kind = "image" }
end

return resizerMover
