local lgr = love.graphics
local tmr = love.timer

local threeDeeGrid = {}

local function getNoise(x, y)
	local n = love.math.noise(x / 8, y / 8) * 800
	return n
end

function threeDeeGrid:init()
	lgr.setLineJoin("miter")
	self.points = {}
	self.spread = 800
	self.speed = 800 * 3

	self.gridWidth = 40
	self.gridDistance = 30
	self.points["horiz1"] = { x = -10000, z = self.gridDistance * self.spread, y = 0 }
	self.points["horiz2"] = { x = self.spread * (self.gridWidth + 1) + 10000, z = self.gridDistance * self.spread, y = 0 }
	for z = 1, 40 do
		for x = 0, self.gridWidth do
			self.points[x.."_"..z] = { x = x * self.spread, z = z * self.spread, y = 0 }
			-- flux.to(self.points[x.."_"..z], 3, { y = 0 }):delay(z * 0.1)
		end
	end
	self.count = 1
	self.driven = 0
	self.distance = 0

	local div = RENDERWIDTH / 8
	-- local cnt = 0
	-- local sh = 0
	-- local dir = -1
	-- local peak = {}
	-- while cnt < div do
	-- 	local r = math.random(2,5)
	-- 	sh = sh + r * dir
	-- 	cnt = cnt + r
	-- 	print(cnt, sh, dir)
	-- 	if dir == -1 then dir = 1 else dir = -1 end
	-- end
	local peak = {}
	local n = math.random()
	for i = 0, div do
		peak[i] = -((love.math.noise(n, i * 100)) * 30 + 10)
	end
	self.mountain = {}
	for i = 0, #peak-1 do
		self.mountain[i] = {
			i * 16,
			peak[i],
			(i+1) * 16,
			peak[i+1],
			(i+1) * 16,
			0,
			i * 16,
			0,
		}
	end


	self.focal = 400

	self.furthestDistance = 0

	self.risers = {}

	self:setupCamera { x = self.spread * (((self.gridWidth + 1) / 2)), y = -2000, z = -1000, tilty = 6, tiltx = 0}
end

function threeDeeGrid:update(dt)
	-- self:setCameraPosition(self.camera.x + dt * 1000, -500, self.camera.z + dt * 1000)
	-- self.camera.tiltx = math.sin(tmr.getTime()) * 7
	-- self.camera.tilty = 6 + math.sin(tmr.getTime()) * 1
	-- self.camera.y = -2000 + math.sin(tmr.getTime()) * 3

	self.distance = self.distance + dt * self.speed
	self.driven = self.driven + dt * self.speed
	local picked = false
	if self.driven >= self.spread then
		self.driven = self.driven - self.spread
		for x = 0, self.gridWidth do
			self.points[x.."_"..self.count] = nil
			self.points[x.."_"..(self.count + self.gridDistance)] = { x = x * self.spread, z = (self.gridDistance) * self.spread, y = 0 }
			-- flux.to(self.points[x.."_"..(self.count + self.gridDistance)], 3, { y = -getNoise(x, self.count) })
		end
		-- if math.random() > 0.6 and not picked then
		-- 	local z = self.gridDistance - 1
		-- 	local x = math.random(14,25)
		-- 	local p1, p2 = "riser_"..self.count.."_"..x.."_"..z, "riser_"..self.count.."_"..(x+1).."_"..z
		-- 	local p3, p4 = "riser_"..self.count.."_"..(x+1).."_"..(z+1), "riser_"..self.count.."_"..x.."_"..(z+1)
		-- 	local p1b, p2b = "riser_l"..self.count.."_"..x.."_"..z, "riser_l"..self.count.."_"..(x+1).."_"..z
		-- 	local p3b, p4b = "riser_l"..self.count.."_"..(x+1).."_"..(z+1), "riser_l"..self.count.."_"..x.."_"..(z+1)
		-- 	-- local p1b, p2b = x.."_"..z, (x+1).."_"..z
		-- 	-- local p3b, p4b = (x+1).."_"..(z+1), x.."_"..(z+1)
		-- 	local h = -800--getNoise(x, self.count)
		-- 	local n = #self.risers+1
		-- 	self.points[p1] = { x = x * self.spread, z = z * self.spread, y = 0, riser = n }
		-- 	self.points[p2] = { x = (x+1) * self.spread, z = z * self.spread, y = 0, riser = n }
		-- 	self.points[p3] = { x = x * self.spread, z = (z+1) * self.spread, y = 0, riser = n }
		-- 	self.points[p4] = { x = (x+1) * self.spread, z = (z+1) * self.spread, y = 0, riser = n }
		-- 	self.points[p1b] = { x = x * self.spread, z = z * self.spread, y = 0, riser = n }
		-- 	self.points[p2b] = { x = (x+1) * self.spread, z = z * self.spread, y = 0, riser = n }
		-- 	self.points[p3b] = { x = x * self.spread, z = (z+1) * self.spread, y = 0, riser = n }
		-- 	self.points[p4b] = { x = (x+1) * self.spread, z = (z+1) * self.spread, y = 0, riser = n }
		-- 	flux.to(self.points[p1], 2, { y = h })
		-- 	flux.to(self.points[p2], 2, { y = h })
		-- 	flux.to(self.points[p3], 2, { y = h })
		-- 	flux.to(self.points[p4], 2, { y = h })

		-- 	self.risers[n] = { p1, p2, p3, p4, p1b, p2b, p3b, p4b }
		-- 	picked = true
		-- end
		self.count = self.count + 1
	end

	self:calculateCameraRotation(self.camera)

	local pointsToRemove = {}
	for i, p in pairs(self.points) do
		if string.sub(i, 1, 5) == "riser" then
			p.z = p.z - self.speed * dt
			if p.z < 400 then
				pointsToRemove[#pointsToRemove+1] = i
			end
		elseif i ~= "horiz1" and i ~= "horiz2" then
			p.z = p.z - self.speed * dt
		end
	end
	for i = 1, #pointsToRemove do
		self.points[pointsToRemove[i]] = nil
	end

	self.calcPoints = {}
	for i, p in pairs(self.points) do
		local cx, cy, dist, scale, onScreen = self:point3dto2d(p.x, p.y, p.z, self.camera)
		if dist > self.furthestDistance then self.furthestDistance = dist end
		self.calcPoints[i] = { cx = cx, cy = cy, dist = dist, scale = scale, onScreen = onScreen }
	end
	table.sort(self.calcPoints, function(a, b) return a.dist < b.dist end)

	self.calcLines = {}
	self.calcPolys = {}

	local p = self.calcPoints
	local c = {0,255,255-(p["horiz1"].scale * 10)}
	local x1, y1 = p["horiz1"].cx, p["horiz1"].cy
	local x2, y2 = p["horiz2"].cx, p["horiz2"].cy
	local n = #self.calcLines+1
	self.calcLines[n] = { x1 = x1, y1 = y1, x2 = x2, y2 = y2, color = {255,0,255} }
	self.horizonX1 = x1
	self.horizonX2 = x2
	self.horizonY1 = y1
	self.horizonY2 = y2

	for x = 0, self.gridWidth-1 do
		for z = self.count, self.count + self.gridDistance - 2 do
			local p = self.calcPoints
			local m = (p[x.."_"..(z+1)].dist - 2000) / 100
			local c = {m,255-m,255}

			local TL = p[x.."_"..(z+1)]
			local TR = p[(x+1).."_"..(z+1)]
			local BL = p[x.."_"..z]
			local BR = p[(x+1).."_"..z]

			local bf = not self:backface(TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy, BL.cx, BL.cy)
			if self:onScreen(TL, TR, BR, BL, RENDERWIDTH * 2, RENDERHEIGHT * 2, false) and not bf then
				self.calcPolys[#self.calcPolys+1] = {
					points = {TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy, BL.cx, BL.cy},
					color = c, dist = TL.dist - 100000
				}
			end
			-- local bf = self:backface(TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy)
			-- if self:onScreen(TL, TR, BR, nil, RENDERWIDTH * 2, RENDERHEIGHT * 2, false) and not bf then
			-- 	self.calcPolys[#self.calcPolys+1] = {
			-- 		points = {TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy},
			-- 		lines = { false, true, true },
			-- 		color = c
			-- 	}
			-- end
			-- local bf = self:backface(TL.cx, TL.cy, BL.cx, BL.cy, BR.cx, BR.cy)
			-- if self:onScreen(TL, BL, BR, nil, RENDERWIDTH * 2, RENDERHEIGHT * 2, false) then
			-- 	self.calcPolys[#self.calcPolys+1] = {
			-- 		points = {TL.cx, TL.cy, BL.cx, BL.cy, BR.cx, BR.cy},
			-- 		lines = { false, true, true },
			-- 		color = c
			-- 	}
			-- end

			-- local x1, y1 = p[x.."_"..(z+1)].cx, p[x.."_"..(z+1)].cy
			-- local x2, y2 = p[(x+1).."_"..(z+1)].cx, p[(x+1).."_"..(z+1)].cy
			-- local n = #self.calcLines+1
			-- self.calcLines[n] = { x1 = TL.cx, y1 = TL.cy, x2 = TR.cx, y2 = TR.cy, color = c }

			-- local x1, y1 = p[x.."_"..z].cx, p[x.."_"..z].cy
			-- local x2, y2 = p[x.."_"..(z+1)].cx, p[x.."_"..(z+1)].cy
			-- local n = #self.calcLines+1
			-- self.calcLines[n] = { x1 = TL.cx, y1 = TL.cy, x2 = BL.cx, y2 = BL.cy, color = c }
		end
	end

	for i = 1, #self.risers do
		local r = self.risers[i]
		local p = self.calcPoints
		if p[r[1]] then
			local m = ((p[r[1]].dist) - 2000) / 100
			local c = {m,255-m,0}

			local TL = p[r[1]]
			local TR = p[r[2]]
			local BL = p[r[3]]
			local BR = p[r[4]]

			local TLb = p[r[5]]
			local TRb = p[r[6]]
			local BLb = p[r[7]]
			local BRb = p[r[8]]

			local bf = not self:backface(TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy, BL.cx, BL.cy)
			if self:onScreen(TL, TR, BR, BL, RENDERWIDTH * 2, RENDERHEIGHT * 2, false) and bf then
				self.calcPolys[#self.calcPolys+1] = {
					points = {TR.cx, TR.cy, BR.cx, BR.cy, BRb.cx, BRb.cy, TRb.cx, TRb.cy},
					color = c, dist = BL.dist
				}
			end

			local bf = not self:backface(TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy, BL.cx, BL.cy)
			if self:onScreen(TL, TR, BR, BL, RENDERWIDTH * 2, RENDERHEIGHT * 2, false) and bf then
				self.calcPolys[#self.calcPolys+1] = {
					points = {BL.cx, BL.cy, TL.cx, TL.cy, TLb.cx, TLb.cy, BLb.cx, BLb.cy},
					color = c, dist = BL.dist
				}
			end

			local bf = not self:backface(TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy, BL.cx, BL.cy)
			if self:onScreen(TL, TR, BR, BL, RENDERWIDTH * 2, RENDERHEIGHT * 2, false) and bf then
				self.calcPolys[#self.calcPolys+1] = {
					points = {TL.cx, TL.cy, TR.cx, TR.cy, TRb.cx, TRb.cy, TLb.cx, TLb.cy},
					color = c, dist = BL.dist
				}
			end

			local bf = not self:backface(TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy, BL.cx, BL.cy)
			if self:onScreen(TL, TR, BR, BL, RENDERWIDTH * 2, RENDERHEIGHT * 2, false) and bf then
				self.calcPolys[#self.calcPolys+1] = {
					points = {TL.cx, TL.cy, TR.cx, TR.cy, BR.cx, BR.cy, BL.cx, BL.cy},
					color = c, dist = BL.dist
				}
			end
		else
			for i = 1, 4 do
				p[r[i]] = nil
			end
		end
	end

	table.sort(self.calcPolys, function(a, b) return a.dist < b.dist end)
end

function threeDeeGrid:draw()
	if self.calcPolys then
		for i = 0, #self.mountain do
			lgr.push()
			lgr.translate(0, self.horizonY1)
			lgr.setColor(255,0,255)
			lgr.polygon("fill", self.mountain[i])
			lgr.setColor(255, 100 * titleState.sunOffsetY, 255)
			lgr.setLineWidth(2)
			lgr.line(self.mountain[i][1], self.mountain[i][2], self.mountain[i][3], self.mountain[i][4])
			lgr.setLineWidth(1)
			lgr.pop()
		end

		lgr.setColor(0,0,30)
		lgr.polygon("fill", self.horizonX1, self.horizonY1, self.horizonX2, self.horizonY2, RENDERWIDTH, RENDERHEIGHT, 0, RENDERHEIGHT)

		for i = 0, #self.mountain do
			lgr.push()
			lgr.translate(0, self.horizonY1)
			lgr.scale(1,-1)
			lgr.setColor(255,0,255,100)
			lgr.polygon("fill", self.mountain[i])
			lgr.pop()
		end

		lgr.setColor(255,0,255)
		lgr.polygon("fill", self.horizonX1, self.horizonY1, self.horizonX2, self.horizonY2, self.horizonX2, self.horizonY2 + 3, self.horizonX1, self.horizonY1 + 3)

		for i, p in pairs(self.calcPolys) do
			-- lgr.setColor(0,10,0)
			-- lgr.polygon("fill", p.points)
			lgr.setColor(p.color)
			lgr.polygon("line", p.points)
			-- if p.lines[1] then
			-- 	lgr.setColor(p.color)
			-- 	lgr.line(p.points[1], p.points[2], p.points[3], p.points[4])
			-- elseif p.lines[2] then
			-- 	lgr.setColor(p.color)
			-- 	lgr.line(p.points[3], p.points[4], p.points[5], p.points[6])
			-- elseif p.lines[3] then
			-- 	lgr.setColor(p.color)
			-- 	lgr.line(p.points[5], p.points[6], p.points[1], p.points[2])
			-- end
		end
	end

	-- if self.calcLines then
	-- 	for i, l in pairs(self.calcLines) do
	-- 		lgr.setColor(l.color)
	-- 		lgr.line(l.x1, l.y1, l.x2, l.y2)
	-- 	end
	-- end

	-- local cnt = 0
	-- for i, p in pairs(self.calcPoints) do
	-- 	if p.cx and p.cy then
	-- 		if string.sub(i, 1, 5) == "riser" then
	-- 			lgr.setColor(1,1,1)
	-- 			lgr.circle("fill", (p.cx), (p.cy), math.max(p.scale * 10, 1))
	-- 		end
	-- 		cnt = cnt + 1
	-- 	end
	-- end
end



--  dP""b8    db    8b    d8 888888 88""Yb    db
-- dP   `"   dPYb   88b  d88 88__   88__dP   dPYb
-- Yb       dP__Yb  88YbdP88 88""   88"Yb   dP__Yb
--  YboodP dP""""Yb 88 YY 88 888888 88  Yb dP""""Yb
function threeDeeGrid:setupCamera(v)
	local x = v.x or 0
	local y = v.y or 0
	local z = v.z or 0
	local tiltx = v.tiltx or 0
	local tilty = v.tilty or 0
	local tiltz = v.tiltz or 0
	self.camera = {x=x,y=y,z=z, tiltx=tiltx,tilty=tilty,tiltz=tiltz, ptx=-1,pty=-1,ptz=-1, locX=x,locZ=z, focusOnPosition = false}
end

function threeDeeGrid:calculateCameraRotation(cam)
	if cam.pty ~= cam.tilty then
		cam.sx = math.sin(math.rad(cam.tilty))
		cam.cx = math.cos(math.rad(cam.tilty))
	end
	if cam.ptz ~= cam.tiltz then
		cam.sy = math.sin(math.rad(cam.tiltz))
		cam.cy = math.cos(math.rad(cam.tiltz))
	end
	if cam.ptx ~= cam.tiltx then
		cam.sz = math.sin(math.rad(cam.tiltx))
		cam.cz = math.cos(math.rad(cam.tiltx))
	end
	cam.ptx = cam.tiltx
	cam.pty = cam.tilty
	cam.ptz = cam.tiltz
end

function threeDeeGrid:setCameraPosition(x, y, z, tilt)
	self.camera.x = x
	self.camera.y = y
	self.camera.z = z
	self.camera.tilty = tilt or self.camera.tilty
end


-- 888888 88  88 88""Yb 888888 888888     8888b.  888888 888888     888888 88   88 88b 88  dP""b8 888888 88  dP"Yb  88b 88 .dP"Y8
--   88   88  88 88__dP 88__   88__        8I  Yb 88__   88__       88__   88   88 88Yb88 dP   `"   88   88 dP   Yb 88Yb88 `Ybo."
--   88   888888 88"Yb  88""   88""        8I  dY 88""   88""       88""   Y8   8P 88 Y88 Yb        88   88 Yb   dP 88 Y88 o.`Y8b
--   88   88  88 88  Yb 888888 888888     8888Y"  888888 888888     88     `YbodP' 88  Y8  YboodP   88   88  YbodP  88  Y8 8bodP'
function threeDeeGrid:point3dto2d(x, y, z, cam)
	local tt = tmr.getTime()
	local dw = RENDERWIDTH
	local dh = RENDERHEIGHT
	local wss = self.zoomScale or 1
	local xy, xz, yz, yx, zx, zy, scale, dist, rx, ry, rz, xo, yo

	--3D Calculations are done here!!!!
	rx = -cam.locX + x
	ry = -cam.y + y
	rz = -cam.locZ + z

	xy = cam.cx * ry - cam.sx * rz
	xz = cam.sx * ry + cam.cx * rz

	yz = cam.cy * xz - cam.sy * rx
	yx = cam.sy * xz + cam.cy * rx

	zx = cam.cz * yx - cam.sz * xy
	zy = cam.sz * yx + cam.cz * xy

	scale = self.focal / (self.focal + yz)
	dist = yz
	xo = dw / 2
	yo = dh / 2
	rx = xo + (zx * scale) * wss
	ry = yo + (zy * scale) * wss-- + ((dist > 1000 and cam.tilty < 30 and self.bendHorizon) and ((dist - 1000) / self.curveAmount * self.curvature) or 0)

	if dist < -self.focal then
		return huge, huge, 0, 0, false
	end
	-- print(rx, ry, dist, scale, wss)

	return rx, ry, dist, scale * wss, (rx >= 0 and rx <= dw and ry >= 0 and ry <= dh)
end

function threeDeeGrid:backface(x1, y1, x2, y2, x3, y3, x4, y4)
	if not x1 or not x2 or not x3 or not x4 or not y1 or not y2 or not y3 or not y4 then return false end
	return not ((y3-y1)/(x3-x1) - (y2-y1)/(x2-x1) <= 0) ~= ((x1 <= x3) == (x1 > x2))
end

function threeDeeGrid:onScreen(p1, p2, p3, p4, w, h, av)
	p4 = p4 or p1
	local buff = 0
	local minDist = -self.focal
	local maxDist = 30000
	if not p1.cx or not p2.cx or not p3.cx or not p4.cx or not p1.cy or not p2.cy or not p3.cy or not p4.cy then
		return false
	elseif (p1.dist > minDist or p2.dist > minDist or p3.dist > minDist or p4.dist > minDist) and av then
		return true
	elseif  (p1.dist < minDist or p2.dist < minDist or p3.dist < minDist or p4.dist < minDist) or
			(p1.dist > maxDist or p2.dist > maxDist or p3.dist > maxDist or p4.dist > maxDist) then
		return false
	elseif  (p1.cy < 0 and p2.cy < 0 and p3.cy < 0 and p4.cy < 0) or
			(p1.cy > w and p2.cy > w and p3.cy > w and p4.cy > w) or
			(p1.cx < -buff and p2.cx < -buff and p3.cx < -buff and p4.cx < -buff) or
			(p1.cx > h + buff and p2.cx > h + buff and p3.cx > h + buff and p4.cx > h + buff) then
		return false
	else
		return true
	end
end


threeDeeGrid:init()

return threeDeeGrid


