local lgr = love.graphics
local win = love.window
local tmr = love.timer
local lfs = love.filesystem
local kbd = love.keyboard
local string = string
local tonumber = tonumber
local collectgarbage = collectgarbage
local sort = table.sort

threeDeeWorld = {}

function threeDeeWorld:create(v)
	local _i = {}
	setmetatable(_i, {__index = self})
	_i:setup(v or {})
	return _i
end
function threeDeeWorld:setup(v)
	self.drawWidth = v.drawWidth
	self.drawHeight = v.drawHeight

	self.loadedMap = ""
end

-- 88   88 88""Yb 8888b.     db    888888 888888
-- 88   88 88__dP  8I  Yb   dPYb     88   88__
-- Y8   8P 88"""   8I  dY  dP__Yb    88   88""
-- `YbodP' 88     8888Y"  dP""""Yb   88   888888
function threeDeeWorld:update(dt)
	if self.loadedMap == "" then return false end

end


-- 8888b.  88""Yb    db    Yb        dP
--  8I  Yb 88__dP   dPYb    Yb  db  dP
--  8I  dY 88"Yb   dP__Yb    YbdPYbdP
-- 8888Y"  88  Yb dP""""Yb    YP  YP
function threeDeeWorld:draw()
	if self.loadedMap == "" then
		lgr.setFont(font.dialogbold)
		lgr.print("No Map Loaded", 4, 4)
		return false
	end

	lgr.setFont(font.dialogbold)
	lgr.print("Map: " .. self.loadedMap, 4, 4)

	if kbd.isDown("rshift") then
		local y = 0
		for i, p in pairs(self.points) do
			lgr.print(p, 4, y * 8)
			y = y + 1
		end

		-- for x = 1, #self.mapData do
		-- 	for y = 1, #self.mapData[1] do
		-- 		if self.mapData[x][y] == 1 then
		-- 			lgr.setColor(1,1,1)
		-- 			lgr.rectangle("fill", (x-1) * 16, (y-1) * 16, 15, 15)
		-- 		end
		-- 	end
		-- end
	end
end

function threeDeeWorld:loadLevel(map)
	if not lfs.getInfo("maps/"..map..".lua") then return false end

	local pl, sl, md = lfs.load("maps/"..map..".lua")()

	self.mapData = md

	self.points = {}
	self.polys = {}

	self.pointCache = {}

	for i = 1, #pl do
		self:addPolygon(pl[i].points, pl[i].color, pl[i].texture, pl[i].layer, pl[i].flip, pl[i].sortPoint, pl[i].dim or 0)
	end

	self.loadedMap = map
	return true
end




--    db    8888b.  8888b.      88""Yb  dP"Yb  88     Yb  dP  dP""b8  dP"Yb  88b 88
--   dPYb    8I  Yb  8I  Yb     88__dP dP   Yb 88      YbdP  dP   `" dP   Yb 88Yb88
--  dP__Yb   8I  dY  8I  dY     88"""  Yb   dP 88  .o   8P   Yb  "88 Yb   dP 88 Y88
-- dP""""Yb 8888Y"  8888Y"      88      YbodP  88ood8  dP     YboodP  YbodP  88  Y8
function threeDeeWorld:addPolygon(p, c, image, layer, flip, sortPoint, dim)
	local layer = layer or 0
	local flip = flip or false
	local sortPoint = sortPoint or nil
	local c = c or nil
	local image = image or nil
	local sp = {}
	sp[1] = "p" .. p[1][1] .. "_" .. p[1][2] .. "_" .. p[1][3]
	sp[2] = "p" .. p[2][1] .. "_" .. p[2][2] .. "_" .. p[2][3]
	sp[3] = "p" .. p[3][1] .. "_" .. p[3][2] .. "_" .. p[3][3]
	sp[4] = "p" .. p[4][1] .. "_" .. p[4][2] .. "_" .. p[4][3]
	local pts = {}
	for i = 1, 4 do
		if not self.pointCache[sp[i]] then
			local n = #self.points+1
			self.points[n] = {
				x = p[i][1],
				y = p[i][2],
				z = p[i][3],
			}
			-- if self.worldWidth < p[i][1] and p[i][2] == 0 then
			-- 	self.worldWidth = p[i][1]
			-- end
			-- if self.worldDepth < p[i][3] and p[i][2] == 0 then
			-- 	self.worldDepth = p[i][3]
			-- end
			pts[i] = n
			local s = tostring("p" .. p[i][1] .. "_" .. p[i][2] .. "_" .. p[i][3])
			self.pointCache[s] = n
		else
			pts[i] = self.pointCache[sp[i]]
		end
	end
	local n = #self.polys+1
	self.polys[n] = { points = pts, image = image, layer = layer, chunks = {}, flip = flip, sortPoint = sortPoint }
	if c then
		self.polys[n].color = c
	end
	if dim then
		self.polys[n].dim = dim
	end
end





-- 1688888 88  88 88""Yb 888888 888888     8888b.  888888 888888     888888 88   88 88b 88  dP""b8 888888 88  dP"Yb  88b 88 .dP"Y8
--   88   88  88 88__dP 88__   88__        8I  Yb 88__   88__       88__   88   88 88Yb88 dP   `"   88   88 dP   Yb 88Yb88 `Ybo."
--   88   888888 88"Yb  88""   88""        8I  dY 88""   88""       88""   Y8   8P 88 Y88 Yb        88   88 Yb   dP 88 Y88 o.`Y8b
--   88   88  88 88  Yb 888888 888888     8888Y"  888888 888888     88     `YbodP' 88  Y8  YboodP   88   88  YbodP  88  Y8 8bodP'
function threeDeeWorld:point3dto2d(x, y, z, cam)
	local tt = tmr.getTime()
	local t = self.timer
	local dw = tonumber(screen.drawWidth or lgr.getWidth())
	local dh = tonumber(screen.drawHeight or lgr.getHeight())
	local wss = self.zoomScale
	local xy, xz, yz, yx, zx, zy, scale, dist, rx, ry, rz, xo, yo

	local xoff = 0
	local yoff = 0
	local zoff = 0
	-- if y == 32 and z - cam.locZ < 1700 then
	-- 	yoff = math.cos(x + t * 0.5) * math.sin(z + t * 1.7) * 4
	-- end
	if y == 31 then
		yoff = math.cos(x + t * 0.5) * math.sin(z + t * 1.7) * 2
	end
	if y == 33 then
		-- xoff = math.cos(x + t * 0.3) * math.sin(z + t * 0.5) * 10
		zoff = math.cos(x + t * 0.4) * math.sin(z + t * 1) * 10
		yoff = math.cos(x + t * 0.5) * math.sin(z + t * 1.7) * 2
	end
	if y == 60 then
		xoff = math.cos(x + t * 0.5) * math.sin(z + t * 1.7) * 2
		zoff = math.cos(x + t * 1) * math.sin(z + t * 0.7) * 5
	end
	if y == 40 then
		xoff = math.cos(x + t * 0.5) * math.sin(z + t * 1.7) * 2
		zoff = math.cos(x + t * 1) * math.sin(z + t * 0.7) * 5
		yoff = math.cos(x + t * 1.03) * math.sin(z + t * 0.6) * 10
	end

	--3D Calculations are done here!!!!
	rx = -cam.locX + x - xoff
	ry = -cam.y + y - yoff
	rz = -cam.locZ + z - zoff

	xy = cam.cx * ry - cam.sx * rz
	xz = cam.sx * ry + cam.cx * rz

	yz = cam.cy * xz - cam.sy * rx
	yx = cam.sy * xz + cam.cy * rx

	zx = cam.cz * yx - cam.sz * xy
	zy = cam.sz * yx + cam.cz * xy

	scale = self.focal / (self.focal + yz)
	dist = yz
	xo = dw / 2
	yo = dh / 2
	rx = xo + (zx * scale) * wss
	ry = yo + (zy * scale) * wss + ((dist > 1000 and cam.tilty < 30 and self.bendHorizon) and ((dist - 1000) / self.curveAmount * self.curvature) or 0)

	performanceGraph:addToCalcTime(tmr.getTime() - tt)
	if dist < -self.focal then
		return huge, huge, 0, 0, false
	end

	return rx, ry, dist, scale * wss, (rx >= 0 and rx <= dw and ry >= 0 and ry <= dh)
end

function threeDeeWorld:backface(x1, y1, x2, y2, x3, y3, x4, y4)
	if not x1 or not x2 or not x3 or not x4 or not y1 or not y2 or not y3 or not y4 then return false end
	return not ((y3-y1)/(x3-x1) - (y2-y1)/(x2-x1) <= 0) ~= ((x1 <= x3) == (x1 > x2))
end

function threeDeeWorld:onScreen(p1, p2, p3, p4, w, h, av)
	p4 = p4 or p1
	local buff = 0
	local minDist = -self.focal
	local maxDist = self.maxDistance + self.chunkSize
	if not p1.cx or not p2.cx or not p3.cx or not p4.cx or not p1.cy or not p2.cy or not p3.cy or not p4.cy then
		return false
	elseif (p1.dist > minDist or p2.dist > minDist or p3.dist > minDist or p4.dist > minDist) and av then
		return true
	elseif  (p1.dist < minDist or p2.dist < minDist or p3.dist < minDist or p4.dist < minDist) or
			(p1.dist > maxDist or p2.dist > maxDist or p3.dist > maxDist or p4.dist > maxDist) then
		return false
	elseif  (p1.cy < 0 and p2.cy < 0 and p3.cy < 0 and p4.cy < 0) or
			(p1.cy > w and p2.cy > w and p3.cy > w and p4.cy > w) or
			(p1.cx < -buff and p2.cx < -buff and p3.cx < -buff and p4.cx < -buff) or
			(p1.cx > h + buff and p2.cx > h + buff and p3.cx > h + buff and p4.cx > h + buff) then
		return false
	else
		return true
	end
end
