local lgr = love.graphics
local win = love.window
local kbd = love.keyboard
local lms = love.mouse

setupScreen = {}

function setupScreen:load(arg)
	local arg = arg or {}
	self.loaded = true
	self.backgroundUpdate = false
	self.backgroundDraw = false
	self.width = 342
	self.height = 192

	-- texture = require "library.Perspective"

	self.name = "Setup Screen"

	self.resolutionOptions = getResolutionList()
	self.resChoice = 1

	for i = 1, #self.resolutionOptions do
		local r = self.resolutionOptions[i]
		if r.w == preferences.renderwidth and r.h == preferences.renderheight then
			self.resChoice = i
		end
	end

	if CUR_GAME then
		self:setScreen()
	end
end

function setupScreen:update(dt)
	lms.setCursor(appCursors.normal)
end

function setupScreen:draw()
	lgr.push()
	lgr.scale(1 * win.getDPIScale())
	lgr.setFont(font.mono1)
	lgr.setColor(1,1,1)

	lgr.print("CHOOSE:", 32, 32)
	lgr.print("1) Adventure Game", 32, 48)
	lgr.print("2) Prototype List", 32, 64)

	lgr.setFont(font.dialog)
	-- for i = 1, #self.resolutionOptions do
	-- 	local y = 3 + 10 * i
	-- 	local r = self.resolutionOptions[i]
	-- 	lgr.setColor(1,1,1)
	-- 	if i == self.resChoice then
	-- 		lgr.setColor(0,255,255)
	-- 		lgr.rectangle("fill", 0, y - 1, RENDERWIDTH, 10)
	-- 		lgr.setColor(0,0,0)
	-- 	end
	-- 	lgr.print(r.w .. " x " .. r.h, 8, y)
	-- 	lgr.printf(math_round(r.w / TILESIZE, 2) .. " x " .. math_round(r.h / TILESIZE, 2) .. " tiles", 128, y, 128, "center")
	-- 	local aspect = tostring(math_round(r.w / r.h, 5))
	-- 	if aspect == "1.77778" then aspect = "16:9" end
	-- 	if aspect == "1.6" then aspect = "16:10" end
	-- 	if aspect == "1" then aspect = "Square" end
	-- 	lgr.printf(aspect, 80, y, 40, "center")
	-- 	lgr.printf(r.nickname or "", 0, y, RENDERWIDTH - 8, "right")
	-- end

	lgr.pop()
end

function setupScreen:keypressed(k, r)
	if k == "1" then
		CUR_GAME = "game_adventure"
		self:setScreen()
	elseif k == "2" then
		CUR_GAME = "game_proto"
		self:setScreen()
	elseif k == "3" then
		CUR_GAME = "game_window"
		self:setScreen()
	end
	-- if k == "space" or k == "return" then
	-- 	self:setScreen()
	-- elseif k == "up" then
	-- 	self.resChoice = self.resChoice - 1
	-- 	if self.resChoice < 1 then self.resChoice = #self.resolutionOptions end
	-- elseif k == "down" then
	-- 	self.resChoice = self.resChoice + 1
	-- 	if self.resChoice > #self.resolutionOptions then self.resChoice = 1 end
	-- end
end

function setupScreen:unload()
	self.loaded = false
end

function setupScreen:blur(arg)
	local arg = arg or {}
	self.focused = false
end

function setupScreen:focus(arg)
	local arg = arg or {}
	self.focused = true
end

function setupScreen:resize(w, h)

end

function setupScreen:setScreen()
	print(">>> Setup Screen:", RENDERWIDTH, RENDERHEIGHT)
	RENDERWIDTH = preferences.renderwidth
	RENDERHEIGHT = preferences.renderheight
	-- RENDERWIDTH = self.resolutionOptions[self.resChoice].w
	-- RENDERHEIGHT = self.resolutionOptions[self.resChoice].h
	stateStack:changeDrawDimensions(RENDERWIDTH, RENDERHEIGHT)

	require (CUR_GAME..'.states.title')

	if TILESIZE then
		print("Chosen Resolution in")
		print("Tile Values: " .. RENDERWIDTH / TILESIZE .. " x " .. RENDERHEIGHT / TILESIZE)
	end
	print()

	print("Setting up screen!")
	win.setMode(1600, 1100, { resizable = true, borderless = false, vsync = true, highdpi = false,
								minwidth = RENDERWIDTH * 3, minheight = RENDERHEIGHT * 3, msaa = 1, fullscreen = false, fullscreentype = "desktop" })

	local dw, dh = win.getDesktopDimensions()

	love.resize()
	PIXELSCALE = win.getDPIScale()
	threeDeeLib = require "library.Perspective"
	threeDeeLib.preload(true)
	stateStack:replace(titleState)
	win.setPosition(dw - (WINDOW_WIDTH), 0, 1)

	local limits = love.graphics.getSystemLimits()

	print("\nGPU Info:")
	local name, version, vendor, device = love.graphics.getRendererInfo()
	print(string.format("  Name: %s \n  Version: %s \n  Vendor: %s \n  Device: %s", name, version, vendor, device))

	local function isWidescreen(w, h)
		if w / h == 1.6 then
			return "  (16:10)"
		end
		return ""
	end

	for i = 1, win.getDisplayCount() do
		print("Display: #"..i)
		local width, height = win.getDesktopDimensions(i)
		print("  Name:", win.getDisplayName(i))
		print("  Dimensions:", width .. " x " .. height)
		local rmode = ""
		if win.getDPIScale() == 2 then rmode = "(Retina Mode)" end
		print("  Pixel Scale:", win.getDPIScale(), rmode)

		-- print("Available Fullscreen modes:")
		-- local modes = win.getFullscreenModes(i)
		-- table.sort(modes, function(a, b) return a.width*a.height < b.width*b.height end)
		-- for j = 1, #modes do
		-- 	local m = modes[j]
		-- 	print("  " .. j .. ") " .. m.width .. " x " .. m.height .. isWidescreen(m.width, m.height))
		-- end
	end
	local x, y, display = win.getPosition()
	print("Window Position:", x .. " x " .. y)
	print("Current Display:", display)
	print("Maximum Texture Size:", formatNumber(limits.texturesize) .. " pixels")
	print()

	local state, percent, seconds = love.system.getPowerInfo()
end
