local lgr = love.graphics
local win = love.window
local lfs = love.filesystem
local kbd = love.keyboard
local tmr = love.timer
local lms = love.mouse

vidResOptions = {}

function vidResOptions:load(arg)
	__dprint("Loading Base State")
	local arg = arg or {}
	self.loaded = true
	self.backgroundUpdate = false
	self.backgroundDraw = false
	self.name = "Vid Res Menu"

	self.onlyShowWholeTileDimensions = true

	self.grid = {}
	self.grid.image = lgr.newImage("game_adventure/images/editorui/grid.png")
	self.grid.quad = lgr.newQuad(0, 0, 16, 16, self.grid.image:getWidth(), self.grid.image:getHeight())

	self:setupMenu()

	self.flux = flux:group()
end

function vidResOptions:update(dt)
	lms.setCursor(appCursors.normal)
	self.menu:update(dt)
	self.flux:update(dt)
end

function vidResOptions:draw()
	lgr.setColor(0,0,0,0.54)
	lgr.rectangle("fill", 0, 0, RENDERWIDTH, RENDERHEIGHT)
	lgr.setColor(1,1,1,0.39)
	for x = 0, math.ceil(RENDERWIDTH / TILESIZE) do
		for y = 0, math.ceil(RENDERHEIGHT / TILESIZE) do
			lgr.draw(self.grid.image, self.grid.quad, x * TILESIZE + 8, y * TILESIZE + 8, math.pi, 1, 1, 8, 8)
		end
	end

	self.menu:draw()
end

function vidResOptions:keypressed(k, r)
	self.menu:keypressed(k, r)
end

function vidResOptions:mousepressed(x, y, button)

end


function vidResOptions:unload()
	__dprint("Unloading Base State")
	self.menu:unload()
	self.menu = nil
	self.loaded = false
	PREFS_CHANGED = true
end


function vidResOptions:blur(arg)
	local arg = arg or {}
	self.focused = false
end


function vidResOptions:focus(arg)
	local arg = arg or {}
	self.focused = true
end


function vidResOptions:resize(w, h)
	__dprint("Resize Base State")
end

function vidResOptions:toggleMenu()
	self.onlyShowWholeTileDimensions = not self.onlyShowWholeTileDimensions
	self:setupMenu()
end

function vidResOptions:setupMenu()
	print("Generate Menu...")
	local hPad = RENDERWIDTH < 320 and 0 or 15
	local lh = RENDERWIDTH < 320 and 10 or 12
	local fnt = RENDERWIDTH < 320 and font.tiny or font.dialog
	self.menu = lfs.load("library/gui/gui.lua")()
	self.menu:init { x = 0, y = 0, w = RENDERWIDTH - (hPad * 2), h = RENDERHEIGHT - 12, padLeft = hPad, padRight = hPad, padTop = 6, padBottom = 6, lineHeight = lh, bgColor = {0,0,0,0}, rightWidth = 70,
		defaultFont = fnt, menuSelectionFont = fnt, menuSelectionBGColor = {0,0.78,0.78}, topHeader = "Set Resolution", onescape = function() stateStack:back() end }

	self.menu:addWidget("top", "checkbox", "1", { value = self.onlyShowWholeTileDimensions, label = "Only Show Whole Tile Resolutions", onactivate = function() self:toggleMenu() end })

	self.menu:addWidget("top", "separator", "3", {  })

	self.resolutionOptions = getResolutionList()
	for i = 1, #self.resolutionOptions do
		local o = self.resolutionOptions[i]

		local sel = -1
		local current = false
		if RENDERWIDTH == o.w and RENDERHEIGHT == o.h then
			current = true
			sel = i
		end

		local show = true
		local wn = (isWholeNumber(o.w / TILESIZE) and isWholeNumber(o.h / TILESIZE))
		if self.onlyShowWholeTileDimensions and not wn then
			show = false
		end

		if show then
			self.menu:addWidget("top", "radio", "1", { value = current, displayValues = {"<", ""}, label = o.w .. "x" .. o.h .. " (" .. math_round(o.w / TILESIZE, 2) .. "x" .. math_round(o.h / TILESIZE, 2) .. (RENDERWIDTH < 200 and "" or " tiles") .. " / " .. o.aspect .. ")", subLabel = (RENDERWIDTH > 150 and o.nickname or ""), onactivate = function(val)
				if val then
					RENDERWIDTH = o.w
					RENDERHEIGHT = o.h
					stateStack:changeDrawDimensions(RENDERWIDTH, RENDERHEIGHT)
					vidResOptions:setupMenu()
					-- self.menu:changeDimensions(o.w - 30, o.h - 12)
					titleState:setupMenu()
					titleState.menu:setPage("vidoptions")
					-- titleState.menu:changeDimensions(nil, o.h - 12)
					titleState:setupBackground()
				end
			end })
		end
	end
	self.menu:addWidget("top", "separator", "3", {  })
	self.menu:addWidget("top", "button", "2", { label = "Back", onactivate = function() stateStack:back() end })
end
